<?php

namespace Bluebirdday\TranssmartSmartConnect\Block\Adminhtml\Form\Field;

use Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Booking\Profile\CollectionFactory as ProfileCollectionFactory;
use Magento\Framework\Data\Form\Element\Factory as ElementFactory;
use Magento\Backend\Block\Template\Context;
use Magento\Config\Block\System\Config\Form\Field\FieldArray\AbstractFieldArray;

/**
 * Class ProfileMapping
 * @package Bluebirdday\TranssmartSmartConnect\Block\Adminhtml\Form\Field
 */
class ProfilesTable extends AbstractFieldArray
{
    /**
     * @var ProfileCollectionFactory
     */
    protected $profileCollectionFactory;

    /**
     * @var ElementFactory
     */
    protected $elementFactory;

    /**
     * @var \Magento\Config\Model\Config\Source\Yesno
     */
    private $yesnoModel;

    /**
     * @var DropdownFactory
     */
    private $dropdownFactory;

    /**
     * @var string
     */
    private $buttonLabel;

    /**
     * ProfileMapping constructor.
     *
     * @param ProfileCollectionFactory $profileCollectionFactory
     * @param ElementFactory $elementFactory
     * @param Context $context
     * @param \Magento\Config\Model\Config\Source\Yesno $yesnoModel
     * @param DropdownFactory $dropdownFactory
     * @param array $data
     * @param string $buttonLabel
     */
    public function __construct(
        ProfileCollectionFactory $profileCollectionFactory,
        ElementFactory $elementFactory,
        Context $context,
        \Magento\Config\Model\Config\Source\Yesno $yesnoModel,
        DropdownFactory $dropdownFactory,
        array $data = [],
        $buttonLabel = 'Add'
    ) {
        $this->profileCollectionFactory = $profileCollectionFactory;
        $this->elementFactory = $elementFactory;
        $this->yesnoModel = $yesnoModel;
        $this->dropdownFactory = $dropdownFactory;
        $this->buttonLabel = $buttonLabel;
        parent::__construct($context, $data);
    }

    /**
     * @param string $columnName
     * @return Dropdown
     */
    private function createIsActiveRenderer($columnName)
    {
        $yesnoElement = $this->elementFactory->create('select');
        $yesnoElement->setForm(
            $this->getForm()
        )->setName(
            $this->_getCellInputElementName($columnName)
        )->setHtmlId(
            $this->_getCellInputElementId('<%- _id %>', $columnName)
        )->setValues(
            $this->yesnoModel->toOptionArray()
        );

        $activeRenderer = $this->dropdownFactory->create(['element' => $yesnoElement]);
        return $activeRenderer;
    }

    private function createProfileRenderer($columnName)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Booking\Profile\Collection $collection */
        $collection = $this->profileCollectionFactory->create();
        $options = $collection->toOptionArray();
        $profileElement = $this->elementFactory->create('select');
        $profileElement->setForm(
            $this->getForm()
        )->setName(
            $this->_getCellInputElementName($columnName)
        )->setHtmlId(
            $this->_getCellInputElementId('<%- _id %>', $columnName)
        )->setValues(
            $options
        );
        $profileRenderer = $this->dropdownFactory->create(['element' => $profileElement]);
        return $profileRenderer;
    }

    /**
     * @inheritdoc
     */
    protected function _prepareToRender()
    {
        $activeColName = 'is_active';
        $activeRenderer = $this->createIsActiveRenderer($activeColName);
        $this->addColumn(
            $activeColName,
            [
                'label' => 'Active',
                'size' => '15%',
                'renderer' => $activeRenderer,
            ]
        );
        $profileColName = 'carrier_profile';
        $profileRenderer = $this->createProfileRenderer($profileColName);
        $this->addColumn(
            $profileColName,
            [
                'label' => 'Carrier Profile',
                'size' => '35%',
                'renderer' => $profileRenderer,
            ]
        );
        $this->addColumn(
            'title_override',
            [
                'label' => 'Title',
                'size' => '35%',
            ]
        );
        $this->addColumn(
            'price',
            [
                'label' => 'Price',
                'size' => '15%',
                'class' => 'validate-number',
            ]
        );

        $this->_addAfter = false;
        $this->_addButtonLabel = __($this->buttonLabel);

        parent::_prepareToRender();
    }
}