<?php
namespace Bluebirdday\TranssmartSmartConnect\Block\Adminhtml\Form\Field;

use Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Booking\Profile\CollectionFactory as ProfileCollectionFactory;
use Magento\Framework\Data\Form\Element\Factory as ElementFactory;
use Magento\Backend\Block\Template\Context;
use Magento\Config\Block\System\Config\Form\Field\FieldArray\AbstractFieldArray;

/**
 * Map shipping method to SmartConnect carrier profile
 * @package Bluebirdday\TranssmartSmartConnect\Block\Adminhtml\Form\Field
 */
class ShippingMethodMapping extends AbstractFieldArray
{
    /**
     * @var ProfileCollectionFactory
     */
    protected $profileCollectionFactory;

    /**
     * @var ElementFactory
     */
    protected $elementFactory;

    /**
     * @var DropdownFactory
     */
    private $dropdownFactory;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Adminhtml\System\Config\Source\Shipping\Allmethods
     */
    private $allShippingMethods;

    /**
     * ProfileMapping constructor.
     *
     * @param ProfileCollectionFactory $profileCollectionFactory
     * @param ElementFactory $elementFactory
     * @param Context $context
     * @param DropdownFactory $dropdownFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Adminhtml\System\Config\Source\Shipping\Allmethods $allShippingMethods
     * @param array $data
     */
    public function __construct(
        ProfileCollectionFactory $profileCollectionFactory,
        ElementFactory $elementFactory,
        Context $context,
        DropdownFactory $dropdownFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\Adminhtml\System\Config\Source\Shipping\Allmethods $allShippingMethods,
        array $data = []
    ) {
        $this->profileCollectionFactory = $profileCollectionFactory;
        $this->elementFactory = $elementFactory;
        $this->dropdownFactory = $dropdownFactory;
        $this->allShippingMethods = $allShippingMethods;
        parent::__construct($context, $data);
    }

    /**
     * Create shipping method renderer as dropdown and fill it with values
     *
     * @param string $columnName
     * @return Dropdown
     */
    private function createProfileRenderer($columnName)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Booking\Profile\Collection $collection */
        $collection = $this->profileCollectionFactory->create();
        $options = $collection->toOptionArray();
        $profileElement = $this->elementFactory->create('select');
        $profileElement->setForm(
            $this->getForm()
        )->setName(
            $this->_getCellInputElementName($columnName)
        )->setHtmlId(
            $this->_getCellInputElementId('<%- _id %>', $columnName)
        )->setValues(
            $options
        );
        $profileRenderer = $this->dropdownFactory->create(['element' => $profileElement]);
        return $profileRenderer;
    }

    /**
     * Create shipping method renderer as dropdown and fill it with values
     *
     * @param string $columnName
     * @return Dropdown
     */
    private function createShippingMethodRenderer($columnName)
    {
        $options = $this->allShippingMethods->toOptionArray();
        $profileElement = $this->elementFactory->create('select');
        $profileElement->setForm(
            $this->getForm()
        )->setName(
            $this->_getCellInputElementName($columnName)
        )->setHtmlId(
            $this->_getCellInputElementId('<%- _id %>', $columnName)
        )->setValues(
            $options
        );
        $profileRenderer = $this->dropdownFactory->create(['element' => $profileElement]);
        return $profileRenderer;
    }

    /**
     * @inheritdoc
     */
    protected function _prepareToRender()
    {
        $shippingMethodColName = 'shipping_method';
        $shippingMethodRenderer = $this->createShippingMethodRenderer($shippingMethodColName);
        $this->addColumn(
            $shippingMethodColName,
            [
                'label' => 'Shipping Method',
                'size' => '50%',
                'renderer' => $shippingMethodRenderer,
            ]
        );

        $profileColName = 'carrier_profile';
        $profileRenderer = $this->createProfileRenderer($profileColName);
        $this->addColumn(
            $profileColName,
            [
                'label' => 'Carrier Profile',
                'size' => '50%',
                'renderer' => $profileRenderer,
            ]
        );
        $this->_addAfter = false;
        $this->_addButtonLabel = __('Add profile mapping');

        parent::_prepareToRender();
    }
}
