<?php

namespace Bluebirdday\TranssmartSmartConnect\Controller\Adminhtml\Data;

use Magento\Backend\App\Action;
use Psr\Log\LoggerInterface;

class Load extends Action
{
    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\SyncFactory
     */
    private $syncFactory;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\SyncRepository
     */
    private $syncRepository;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\SyncManagement
     */
    private $syncManagement;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Entity\Updater\Pool
     */
    private $updaterPool;

    /**
     * @param Action\Context $context
     * @param LoggerInterface $logger
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Entity\Updater\Pool $updaterPool
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\SyncFactory $syncFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\SyncRepository $syncRepository
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\SyncManagement $syncManagement
     */
    public function __construct(
        Action\Context $context,
        LoggerInterface $logger,
        \Bluebirdday\TranssmartSmartConnect\Model\Entity\Updater\Pool $updaterPool,
        \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\SyncFactory $syncFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\SyncRepository $syncRepository,
        \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\SyncManagement $syncManagement
    ) {
        $this->updaterPool = $updaterPool;
        $this->logger = $logger;
        $this->syncFactory = $syncFactory;
        $this->syncRepository = $syncRepository;
        $this->syncManagement = $syncManagement;
        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\App\Response\Http
     */
    public function execute()
    {
        $result = new \stdClass();
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\Sync $sync */
        $sync = $this->syncFactory->create();
        $sync->setType('list entities');
        $sync->setStatus('started');
        $currentSyncTime = time();
        $sync->setCreatedAt($currentSyncTime);
        $this->syncRepository->save($sync);
        try {
            $this->updaterPool->refresh();
            $sync->setStatus('successful');
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
            $this->getResponse()->setStatusHeader(500, null, __('Cannot load data'));
            $result->error = __('Cannot load data');
            $sync->setStatus('failed');
            $sync->setMessage($e->getMessage());
        }
        $this->syncRepository->save($sync);
        $this->syncManagement->clean();
        $syncId = $sync->getId();
        $updatedSync = $this->syncRepository->load($syncId);
        $result->sync = $updatedSync->getData();
        /** @var \Magento\Framework\App\Response\Http $response */
        $response = $this->getResponse();
        return $response->representJson(json_encode($result));
    }
}
