<?php

namespace Bluebirdday\TranssmartSmartConnect\Controller\Pickup;
use Magento\Framework\Exception\LocalizedException;
use Magento\Store\Model\Information;
use Magento\Directory\Helper\Data as DataHelper;
/**
 * Class Location
 *
 * @package Bluebirdday\TranssmartSmartConnect\Controller\Pickup
 */
class Location extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\Adapter
     */
    private $adapter;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    private $checkoutSession;

    /**
     * @var \Magento\Quote\Model\QuoteRepository
     */
    private $quoteRepository;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Media\FileHandler
     */
    private $fileHandler;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Location\Request\ValidatorFactory
     */
    private $validatorFactory;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Location\RequestFactory
     */
    private $locationRequestFactory;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Rate\Data\Collector
     */
    private $dataCollector;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Street\Address\Parser
     */
    private $streetParser;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Quote\AddressManager
     */
    private $addressManager;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Rate\Response\Parser
     */
    private $rateParser;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * Location constructor.
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Quote\Model\QuoteRepository $quoteRepository
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\Adapter $adapter
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Media\FileHandler $fileHandler
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Location\Request\ValidatorFactory $validatorFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Location\RequestFactory $locationRequestFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Rate\Data\Collector $dataCollector
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Street\Address\Parser $streetParser
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Quote\AddressManager $addressManager
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Rate\Response\Parser $rateParser
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Quote\Model\QuoteRepository $quoteRepository,
        \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\Adapter $adapter,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Bluebirdday\TranssmartSmartConnect\Model\Media\FileHandler $fileHandler,
        \Bluebirdday\TranssmartSmartConnect\Model\Location\Request\ValidatorFactory $validatorFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\Location\RequestFactory $locationRequestFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\Rate\Data\Collector $dataCollector,
        \Bluebirdday\TranssmartSmartConnect\Model\Street\Address\Parser $streetParser,
        \Bluebirdday\TranssmartSmartConnect\Model\Quote\AddressManager $addressManager,
        \Bluebirdday\TranssmartSmartConnect\Model\Rate\Response\Parser $rateParser,
        \Psr\Log\LoggerInterface $logger
    ) {
        parent::__construct($context);
        $this->checkoutSession = $checkoutSession;
        $this->quoteRepository = $quoteRepository;
        $this->scopeConfig = $scopeConfig;
        $this->adapter = $adapter;
        $this->fileHandler = $fileHandler;
        $this->validatorFactory = $validatorFactory;
        $this->locationRequestFactory = $locationRequestFactory;
        $this->dataCollector = $dataCollector;
        $this->streetParser = $streetParser;
        $this->addressManager = $addressManager;
        $this->rateParser = $rateParser;
        $this->logger = $logger;
    }

    /**
     * Prepare data based on customer shipping address to request pickup locations
     *
     * @param string|null $searchString
     * @return array
     */
    private function prepareLocationRequest($searchString)
    {
        $quoteId = $this->checkoutSession->getQuoteId();
        $quote = $this->quoteRepository->getActive($quoteId);
        $estimationAddress = $this->addressManager->getEstimationAddress($quote);
        $destinationAddress =  $estimationAddress ? $estimationAddress : $quote->getShippingAddress();
        $streetToParse = $destinationAddress->getStreet();
        // we use only first address line for location request. see TRANSM2-165
        $firstStreetLine = reset($streetToParse);
        $postCode = $destinationAddress->getPostcode();

        list($street, $houseNumber, $houseNumberExt) = $this->streetParser->parseStreetAddress([$firstStreetLine]);

        $merchantCountry = $this->scopeConfig->getValue(Information::XML_PATH_STORE_INFO_COUNTRY_CODE);
        $countryFrom = $merchantCountry ?: $this->scopeConfig->getValue(DataHelper::XML_PATH_DEFAULT_COUNTRY);

        $requestData = [
            'city' => $destinationAddress->getCity(),
            'zip_code' => $postCode,
            'street' => $street,
            'house_number' => $houseNumber,
            'house_number_ext' => $houseNumberExt,
            'country_from'=> $countryFrom,
            'country_to' => $destinationAddress->getCountryModel()->getData('iso2_code'),
            'provider' => '', // left empty to get all providers
            'limit' => '15', // TODO: could be configured on backend
        ];

        // override values entered in a search string.
        if (!empty($searchString)) {
            // we are looking for <zipcode>,<city>,<street>,<housenumber>
            $searchParts = explode(',', $searchString);
            if (!preg_match("/\d\d\d\d(.*)\w\w/", trim($searchParts[0]))) {
                $searchParts[1] = trim($searchParts[0]);
                $searchParts[0] = '';
            }

            $override = ['zip_code', 'city', 'street', 'house_number'];
            foreach ($override as $index => $fieldName) {
                $requestData[$fieldName] = '';
                if (isset($searchParts[$index]))
                {
                    $requestData[$fieldName] = trim($searchParts[$index]);
                }
            }
        }
        return $requestData;
    }

    /**
     * Return error for proper output on frontend
     *
     * @param string $message
     * @return string
     */
    private function getErrorResponse($message)
    {
        $responseContent = new \stdClass();
        $responseContent->success = false;
        $responseContent->errorMessage = $message;
        return \json_encode($responseContent);
    }

    /**
     * Decode image, save it locally and return its URL
     *
     * @param \stdClass $source
     * @param string $propertyName
     * @param string $prefix
     * @return string
     */
    private function decodeAndSave($source, $propertyName, $prefix)
    {
        $fileUrl = '';

        if (property_exists($source, $propertyName) && !empty($source->$propertyName)) {
            $image = base64_decode($source->$propertyName);
            $imageFileName = $this->fileHandler->save($prefix . '_' . $propertyName, $image);
            $fileUrl = $this->fileHandler->getMediaUrl($imageFileName);
        }

        return $fileUrl;
    }

    /**
     * Return test json response
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\View\Result\Page
     */
    public function execute()
    {
        $searchString = $this->getRequest()->getParam('searchString');
        $locationsRequestData = $this->prepareLocationRequest($searchString);
        $locationsRequest = $this->locationRequestFactory->create(['data' => $locationsRequestData]);
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Location\Request\Validator $validator */
        $validator = $this->validatorFactory->create();
        if (!$validator->isValid($locationsRequest)) {
            $responseContent = $this->getErrorResponse('Invalid request: ' . $validator->getError());
            $this->getResponse()->setHttpResponseCode(400);
            return $this->getResponse()->representJson($responseContent);
        }

        try {
            $activeProfiles = $this->rateParser->getPickupProfiles($locationsRequestData['country_to']);
        } catch (LocalizedException $e) {
            $this->logger->critical($e->getMessage());
            $responseContent = $this->getErrorResponse($e->getMessage());
            $this->getResponse()->setHttpResponseCode($e->getCode());
            return $this->getResponse()->representJson($responseContent);
        }
        try {
            $locations = $this->adapter->getPickupLocations($locationsRequest);
            if ($this->scopeConfig->getValue('transsmart/pickup/dynamic_rates')) {
                $rateRequest = $this->dataCollector->prepareFromAddress();
                $rates = $this->adapter->getRates($rateRequest);
                if (null === $rates || null === $locations) {
                    return $this->getResponse()->representJson(json_encode([]));
                }
                $carrierRates = $this->rateParser->getCarrierRates($rates, $activeProfiles);
            } else {
                $carrierRates = $this->rateParser->getFixedRates($activeProfiles);
            }
        } catch (LocalizedException $exception) {
            $this->logger->alert($exception->getMessage());
            $responseContent = $this->getErrorResponse($exception->getMessage());
            $this->getResponse()->setHttpResponseCode(400);
            return $this->getResponse()->representJson($responseContent);
        } catch (\RuntimeException $exception) {
            $this->logger->alert($exception->getMessage());
            $responseContent = $this->getErrorResponse(__('Cannot perform request. See log for details.'));
            $this->getResponse()->setHttpResponseCode(400);
            return $this->getResponse()->representJson($responseContent);
        }

        /** @var \stdClass[] $sources */
        $sources = $locations->sources;
        foreach ($sources as $sourceIndex => &$source) {
            // if we cannot name the files, we treat them as absent and log error
            if (!property_exists($source, 'name')) {
                $this->logger->error(
                    'Got empty source(carrier) name on Transsmart pickup locations request '
                    . $locationsRequest->__toString()
                );
                $source->logo = '';
                $source->marker = '';
                continue;
            }

            $carrierCode = $source->code;
            if (array_key_exists($carrierCode, $carrierRates)) {
                $source->rates = $carrierRates[$carrierCode];
            } else {
                //when we don't have rates, we don't show this carrier locations
                unset($sources[$sourceIndex]);
               continue;
            }

            try {
                $source->logo = $this->decodeAndSave($source, 'logo', $carrierCode);
                $source->marker = $this->decodeAndSave($source, 'marker', $carrierCode);
            } catch (\Magento\Framework\Exception\FileSystemException $exception) {
                $this->logger->alert($exception->getMessage(), $exception->getTrace());
                $responseContent = $this->getErrorResponse('Cannot save file. See log for details.');
                $this->getResponse()->setHttpResponseCode(500);
                return $this->getResponse()->representJson($responseContent);
            }
        }

        return $this->getResponse()->representJson(json_encode($sources));
    }
}
