# Installation and configuration
## Prerequisites
In this installation and configuration guide we imply that you have Magento 2 already installed. If not, please refer [Magento 2 installation guide](http://devdocs.magento.com/guides/v2.0/install-gde/bk-install-guide.html).
To have this module configured properly you need to have Transsmart credentials.
If you are going to use booking and printing from Transsmart, than you need to follow the same guide in `bluebirdday/transsmartsmartconnect` module. 

## Installation
Connect to the Bluebird Day Transsmart Composer Repository:
```
composer config repositories.transsmart composer https://packages.transsmart.bluebirdday.nl
```

Create a file named 'auth.json' in the root of your project.
Fill it with the code below and replace the username and password credentials with the credentials you got from Transsmart.

```
{
  "http-basic": {
    "packages.transsmart.bluebirdday.nl": {
      "username": "enter username here",
      "password": "enter password here"
    }
  }
}
```

To install the module, please execute the following command:
```
composer require 'bluebirdday/transsmartsmartconnect'
```

To upgrade the module please execute the command:
```
php bin/magento setup:upgrade
```

## Initial Configuration
Basic idea of configuration is to configure your store name, shipping origin, Google Maps key, Transsmart credentials, download basic Transsmart entities and save default values for them.

1. [Optional] Go to *Stores->Settings->Configuration->GENERAL->General->Store Information*, 
Configure your "*Store Name*" and following fields related to your store, then save config.
2. **[Required]** Go to *Stores->Settings->Configuration->SALES->Shipping Settings*, fill your *shipping origin* address fields and save.
3. **[Required]** Go to *Stores->Settings->Configuration->SALES->Transsmart*, open "*Environments*" group, choose your "*Environment to use*" and enter your Transsmart credentials.   
**Important**: click "*Save config*" when the fields are filled, because the following step is based on these values saved to Magento config.
4. **[Required]** Open "*Default Shipment Properties*" group, click "*Load Transsmart data*" button, wait for page refresh. Make sure that successful sync message appeared under the button.
If error message appeared, you need to check your credentials from the previous step.
5. **[Required]** Choose your default profile, email type, incoterm, cost center and package, click "*Save Config*".  
**Important**: even if you are happy with the default values you see, you need to save config to have the system working properly.
6. [Optional] In "*Transsmart pickup locations*" group change Google Maps API key to your own. Refer to [instruction from Google](https://developers.google.com/maps/documentation/geocoding/get-api-key).  
This step becomes *vital* when you *go live* and want to use pickup locations shipping method.
7. [Optional] In "*Transsmart pickup locations*" group add profiles to be enabled for pickup shipping method.
If you enter a title for profile, it will be overridden in the checkout. Field *price* will be utilized only in case when "*Request shipping rates dynamically*" is set to "*No*".  
This step is necessary when you want to use pickup locations shipping method. 
8. [Optional] In "*Shipping Methods*" group add profiles to be displayed as separate shipping methods(delivery options).
If you enter a title for profile, it will be overridden.  
This step is necessary when you want to use delivery options. 
9. [Optional] Go to *Stores->Settings->Configuration->SALES->Transsmart Booking Profiles* tab and configure your profiles individually.
10. [Optional] Configuring table rate:
  * Go to *Stores->Settings->Configuration->SALES->Shipping Methods->Table Rates*;
  * Switch scope to website level;
  * Set "Enabled" to "Yes";
  * Choose "Condition" you want to use;
  * Save config;
  * Press Export CSV;
  * Edit downloaded tablerates.csv file;  
    **Warning!**: Windows editors may leave #FE#FF mark at the beginning of a file to mark it as UTF-8. It is not visible in editor, but it will break parsing this file by Magento.
  * Upload your file in the "Import" line and make sure no errors occurred;
  * Clear cache;

You are ready to use the module!