<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile;
use Magento\Store\Model\ScopeInterface;

class AllowedCountry
{
    const CONFIG_TRANSSMART_PROFILES_SECTION_ID = 'transsmart_profiles';

    /**
     * @var \Magento\Directory\Model\AllowedCountries
     */
    private $allowedCountries;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * AllowedCountry constructor.
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Directory\Model\AllowedCountries $allowedCountries
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Directory\Model\AllowedCountries $allowedCountries
    ) {
        $this->allowedCountries = $allowedCountries;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * Check country configuration per each profile and leave allowed only
     *
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile[] $profiles
     * @param string $countryId two-letter country id
     * @return \stdClass[]
     */
    public function filterProfilesByCountry($profiles, $countryId)
    {
        $filteredProfiles = [];

        foreach ($profiles as $profile) {
            $allowPath = implode(
                '/',
                [
                    self::CONFIG_TRANSSMART_PROFILES_SECTION_ID,
                    $profile->getCode(),
                    'sallowspecific'
                ]
            );
            $allowSpecific = $this->scopeConfig->getValue($allowPath, ScopeInterface::SCOPE_WEBSITE);

            if ($allowSpecific) {
                $countriesPath = implode(
                    '/',
                    [
                        self::CONFIG_TRANSSMART_PROFILES_SECTION_ID,
                        $profile->getCode(),
                        'specificcountry'
                    ]
                );

                $allowedCountries = $this->scopeConfig->getValue($countriesPath, 'websites');
                if ($allowedCountries) {
                    $allowedCountries = explode(',', $allowedCountries);
                }
            } else {
                $allowedCountries = $this->allowedCountries->getAllowedCountries(ScopeInterface::SCOPE_WEBSITE);
            }

            if (is_array($allowedCountries) && in_array($countryId, $allowedCountries)) {
                $filteredProfiles[$profile->getCode()] = $profile;
            }
        }

        return $filteredProfiles;
    }
}
