<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile;

use Magento\Framework\Exception\LocalizedException;

class Updater extends \Bluebirdday\TranssmartSmartConnect\Model\Entity\Updater
{
    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\CarrierRepository
     */
    protected $carrierRepository;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Servicelevel\TimeRepository
     */
    protected $timeRepository;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Servicelevel\OtherRepository
     */
    protected $otherRepository;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\IncotermRepository
     */
    protected $incotermRepository;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\CostcenterRepository
     */
    protected $costcenterRepository;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Mail\TypeRepository
     */
    protected $mailTypeRepository;

    /**
     * Updater constructor.
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\Adapter $adapter
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileRepository $repository
     * @param \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Booking\Profile\Collection $collection
     * @param \Bluebirdday\TranssmartSmartConnect\Model\CarrierRepository $carrierRepository
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Servicelevel\TimeRepository $timeRepository
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Servicelevel\OtherRepository $otherRepository
     * @param \Bluebirdday\TranssmartSmartConnect\Model\IncotermRepository $incotermRepository
     * @param \Bluebirdday\TranssmartSmartConnect\Model\CostcenterRepository $costcenterRepository
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Mail\TypeRepository $mailTypeRepository
     */
    public function __construct(
        \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\Adapter $adapter,
        \Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileRepository $repository,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Booking\Profile\Collection $collection,
        \Bluebirdday\TranssmartSmartConnect\Model\CarrierRepository $carrierRepository,
        \Bluebirdday\TranssmartSmartConnect\Model\Servicelevel\TimeRepository $timeRepository,
        \Bluebirdday\TranssmartSmartConnect\Model\Servicelevel\OtherRepository $otherRepository,
        \Bluebirdday\TranssmartSmartConnect\Model\IncotermRepository $incotermRepository,
        \Bluebirdday\TranssmartSmartConnect\Model\CostcenterRepository $costcenterRepository,
        \Bluebirdday\TranssmartSmartConnect\Model\Mail\TypeRepository $mailTypeRepository
    ) {
        $this->carrierRepository = $carrierRepository;
        $this->timeRepository = $timeRepository;
        $this->otherRepository = $otherRepository;
        $this->incotermRepository = $incotermRepository;
        $this->costcenterRepository = $costcenterRepository;
        $this->mailTypeRepository = $mailTypeRepository;
        parent::__construct($adapter, $repository, $collection, 'getBookingProfiles');
    }

    /**
     * Fill model with data from object
     *
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile $model
     * @param \stdClass $data
     * @throws LocalizedException
     */
    protected function fillModelData($model, $data)
    {
        $model->setCode($data->code);
        $model->setDescription($data->description);
        if ($data->carrier) {
            $carrier = $this->carrierRepository->loadByCode($data->carrier);
            if ($carrier->getId() === null) {
                throw new \InvalidArgumentException('Invalid carrier code: ' . $data->carrier);
            }
            $model->setCarrierId($carrier->getId());
        } else {
            throw new LocalizedException(__('Empty carrier for profile %1', $data->code));
        }


        if ($data->serviceLevelTime) {
            $serviceLevelTime = $this->timeRepository->loadByCode($data->serviceLevelTime);
            if ($serviceLevelTime->getId() === null) {
                throw new \InvalidArgumentException('Invalid service level time: ' . $data->serviceLevelTime);
            }
            $model->setServiceLevelTimeId($serviceLevelTime->getId());
        } else {
            $model->setServiceLevelTimeId(null);
        }

        if ($data->serviceLevelOther) {
            $serviceLevelOther = $this->otherRepository->loadByCode($data->serviceLevelOther);
            if ($serviceLevelOther->getId() === null) {
                throw new \InvalidArgumentException('Invalid service level other: ' . $data->serviceLevelOther);
            }
            $model->setServiceLevelOtherId($serviceLevelOther->getId());
        } else {
            $model->setServiceLevelOtherId(null);
        }

        if ($data->incoterms) {
            $incoterm = $this->incotermRepository->loadByCode($data->incoterms);
            if ($incoterm->getId() === null) {
                throw new \InvalidArgumentException('Invalid incoterm: ' . $data->incoterms);
            }
            $model->setIncotermId($incoterm->getId());
        } else {
            $model->setIncotermId(null);
        }

        if ($data->costCenter) {
            $costcenter = $this->costcenterRepository->loadByCode($data->costCenter);
            if ($costcenter->getId() === null) {
                throw new \InvalidArgumentException('Invalid costCenter: ' . $data->costCenter);
            }
            $model->setCostcenterId($costcenter->getId());
        } else {
            $model->setCostcenterId(null);
        }

        if ($data->mailType) {
            $mailType = $this->mailTypeRepository->loadByCode($data->mailType);
            if ($mailType->getId() === null) {
                throw new \InvalidArgumentException('Invalid mailType: ' . $data->mailType);
            }
            $model->setMailTypeId($mailType->getId());
        } else {
            $model->setMailTypeId(null);
        }
    }
}
