<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Booking;

/**
 * Class ProfileRepository
 * @package Bluebirdday\TranssmartSmartConnect\Model\Booking
 */
class ProfileRepository implements \Bluebirdday\TranssmartSmartConnect\Model\Entity\RepositoryInterface
{
    /**
     * @var ProfileFactory
     */
    private $profileFactory;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Booking\Profile\CollectionFactory
     */
    private $collectionFactory;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Booking\Profile
     */
    private $resourceModel;

    /**
     * ProfileRepository constructor.
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileFactory $profileFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Booking\Profile\CollectionFactory $collectionFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Booking\Profile $resourceModel
     */
    public function __construct(
        \Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileFactory $profileFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Booking\Profile\CollectionFactory $collectionFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Booking\Profile $resourceModel
    ) {
        $this->profileFactory = $profileFactory;
        $this->collectionFactory = $collectionFactory;
        $this->resourceModel = $resourceModel;
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile[]
     */
    public function getList()
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Booking\Profile\Collection $collection */
        $collection = $this->collectionFactory->create();
        return $collection->getItems();
    }

    /**
     * Filter booking profile by codes
     *
     * @param array $codes
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile[]
     */
    public function getByCodes(array $codes)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Booking\Profile\Collection $collection */
        $collection = $this->collectionFactory->create();
        $collection->addFieldToFilter('main_table.code', ['in' => $codes]);
        return $collection->getItems();
    }

    /**
     * @param Profile $profileModel
     */
    public function save($profileModel)
    {
        $this->resourceModel->save($profileModel);
    }

    /**
     * Load booking profile by id
     *
     * @param integer $id
     * @return Profile
     */
    public function load($id)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Profile $model */
        $model = $this->profileFactory->create();
        $this->resourceModel->load($model, $id, null);
        return $model;
    }

    /**
     * Load booking profile by code
     *
     * @param string $code
     * @return Profile
     */
    public function loadByCode($code)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile $model */
        $model = $this->profileFactory->create();
        $this->resourceModel->load($model, $code, 'code');
        return $model;
    }

    /**
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile $profileModel
     */
    public function delete($profileModel)
    {
        $this->resourceModel->delete($profileModel);
    }
}
