<?php

namespace Bluebirdday\TranssmartSmartConnect\Model\Carrier;

use Magento\OfflineShipping\Model\Carrier\Tablerate as BaseTablerate;
use Magento\Quote\Model\Quote\Address\RateRequest;

/**
 * Class Tablerate
 * @package Bluebirdday\TranssmartSmartConnect\Model\Carrier
 */
class Tablerate extends BaseTablerate
{
    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileRepository
     */
    protected $_profileRepository;

    /**
     * Tablerate constructor.
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory $rateErrorFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Shipping\Model\Rate\ResultFactory $rateResultFactory
     * @param \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $resultMethodFactory
     * @param \Magento\OfflineShipping\Model\ResourceModel\Carrier\TablerateFactory $tablerateFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileRepository $profileRepository
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory $rateErrorFactory,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Shipping\Model\Rate\ResultFactory $rateResultFactory,
        \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $resultMethodFactory,
        \Magento\OfflineShipping\Model\ResourceModel\Carrier\TablerateFactory $tablerateFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileRepository $profileRepository,
        array $data = []
    )
    {
        $this->_profileRepository = $profileRepository;
        parent::__construct($scopeConfig, $rateErrorFactory, $logger, $rateResultFactory, $resultMethodFactory, $tablerateFactory, $data);
    }

    /**
     * @inheritdoc
     */
    public function collectRates(RateRequest $request)
    {
        if (!$this->getConfigFlag('active')) {
            return false;
        }

        // exclude Virtual products price from Package value if pre-configured
        if (!$this->getConfigFlag('include_virtual_price') && $request->getAllItems()) {
            foreach ($request->getAllItems() as $item) {
                if ($item->getParentItem()) {
                    continue;
                }
                if ($item->getHasChildren() && $item->isShipSeparately()) {
                    foreach ($item->getChildren() as $child) {
                        if ($child->getProduct()->isVirtual()) {
                            $request->setPackageValue($request->getPackageValue() - $child->getBaseRowTotal());
                        }
                    }
                } elseif ($item->getProduct()->isVirtual()) {
                    $request->setPackageValue($request->getPackageValue() - $item->getBaseRowTotal());
                }
            }
        }

        // Free shipping by qty
        $freeQty = 0;
        if ($request->getAllItems()) {
            $freePackageValue = 0;
            foreach ($request->getAllItems() as $item) {
                if ($item->getProduct()->isVirtual() || $item->getParentItem()) {
                    continue;
                }

                if ($item->getHasChildren() && $item->isShipSeparately()) {
                    foreach ($item->getChildren() as $child) {
                        if ($child->getFreeShipping() && !$child->getProduct()->isVirtual()) {
                            $freeShipping = is_numeric($child->getFreeShipping()) ? $child->getFreeShipping() : 0;
                            $freeQty += $item->getQty() * ($child->getQty() - $freeShipping);
                        }
                    }
                } elseif ($item->getFreeShipping()) {
                    $freeShipping = is_numeric($item->getFreeShipping()) ? $item->getFreeShipping() : 0;
                    $freeQty += $item->getQty() - $freeShipping;
                    $freePackageValue += $item->getBaseRowTotal();
                }
            }
            $oldValue = $request->getPackageValue();
            $request->setPackageValue($oldValue - $freePackageValue);
        }

        if (!$request->getConditionName()) {
            $conditionName = $this->getConfigData('condition_name');
            $request->setConditionName($conditionName ? $conditionName : $this->_defaultConditionName);
        }

        // Package weight and qty free shipping
        $oldWeight = $request->getPackageWeight();
        $oldQty = $request->getPackageQty();

        $request->setPackageWeight($request->getFreeMethodWeight());
        $request->setPackageQty($oldQty - $freeQty);

        /** @var \Magento\Shipping\Model\Rate\Result $result */
        $result = $this->_rateResultFactory->create();
        $rates = $this->getRates($request);

        $request->setPackageWeight($oldWeight);
        $request->setPackageQty($oldQty);

        $count = 0;
        foreach ($rates as $rate) {
            if (!empty($rate) && $rate['price'] >= 0) {
                if ($request->getPackageQty() == $freeQty) {
                    $shippingPrice = 0;
                } else {
                    $shippingPrice = $this->getFinalPriceWithHandlingFee($rate['price']);
                }
                $method = $this->createShippingMethod($shippingPrice, $rate['cost'], $rate['transsmart_bookingprofile_id']);
                $result->append($method);
            }
            $count++;
        }

        if ($count == 0 && $request->getPackageQty() == $freeQty) {

            /**
             * Promotion rule was applied for the whole cart.
             *  In this case all other shipping methods could be omitted
             * Table rate shipping method with 0$ price must be shown if grand total is more than minimal value.
             * Free package weight has been already taken into account.
             */
            $request->setPackageValue($freePackageValue);
            $request->setPackageQty($freeQty);
            $rates = $this->getRates($request);
            foreach ($rates as $rate) {
                if (!empty($rate) && $rate['price'] >= 0) {
                    $method = $this->createShippingMethod(0, 0, $rate['transsmart_bookingprofile_id']);
                    $result->append($method);
                }
                $count++;
            }

        } elseif ($count == 0) {
            /** @var \Magento\Quote\Model\Quote\Address\RateResult\Error $error */
            $error = $this->_rateErrorFactory->create(
                [
                    'data' => [
                        'carrier' => $this->_code,
                        'carrier_title' => $this->getConfigData('title'),
                        'error_message' => $this->getConfigData('specificerrmsg'),
                    ],
                ]
            );
            $result->append($error);
        }

        return $result;
    }

    /**
     * @param RateRequest $request
     * @return array|bool
     */
    public function getRates(RateRequest $request)
    {
        return $this->_tablerateFactory->create()->getRates($request);
    }

    /**
     * Get the method object based on the shipping price and cost.
     *
     * @param float $shippingPrice
     * @param float $cost
     * @param int $profileId
     * @return \Magento\Quote\Model\Quote\Address\RateResult\Method
     */
    private function createShippingMethod($shippingPrice, $cost, $profileId)
    {
        /** @var  \Magento\Quote\Model\Quote\Address\RateResult\Method $method */
        $method = $this->_resultMethodFactory->create();

        $method->setCarrier('tablerate');
        $method->setCarrierTitle($this->getConfigData('title'));

        $method->setMethod($this->getUniqueMethodCode($profileId));
        $method->setMethodTitle($this->getUniqueMethodTitle($profileId));

        $method->setPrice($shippingPrice);
        $method->setCost($cost);

        $method->setTranssmartBookingProfileId($profileId);

        return $method;
    }

    /**
     * Get unique method code.
     *
     * @param int $bookingProfileId
     * @return string
     */
    private function getUniqueMethodCode($bookingProfileId)
    {
        $profile = $this->_profileRepository->load($bookingProfileId);

        return sprintf('bestway_%s', $profile->getCode());
    }

    /**
     * Get unique method title.
     *
     * @param int $bookingProfileId
     * @return false|string
     */
    private function getUniqueMethodTitle($bookingProfileId)
    {
        $profile = $this->_profileRepository->load($bookingProfileId);

        return $profile->getDescription() ?: $this->getConfigData('name');
    }
}