<?php
namespace Bluebirdday\TranssmartSmartConnect\Model;

/**
 * Class CarrierRepository
 * @package Bluebirdday\TranssmartSmartConnect\Model
 */
class CarrierRepository implements \Bluebirdday\TranssmartSmartConnect\Model\Entity\RepositoryInterface
{
    /**
     * @var CarrierFactory
     */
    protected $carrierFactory;

    /**
     * @var Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Carrier\CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Carrier
     */
    protected $resourceModel;

    /**
     * CarrierRepository constructor.
     * @param CarrierFactory $carrierFactory
     * @param Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Carrier\CollectionFactory $collectionFactory
     * @param Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Carrier $resourceModel
     */
    public function __construct(
        \Bluebirdday\TranssmartSmartConnect\Model\CarrierFactory $carrierFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Carrier\CollectionFactory $collectionFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Carrier $resourceModel
    ) {
        $this->carrierFactory = $carrierFactory;
        $this->collectionFactory = $collectionFactory;
        $this->resourceModel = $resourceModel;
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Carrier[]
     */
    public function getList()
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Carrier\Collection $collection */
        $collection = $this->collectionFactory->create();
        return $collection->getItems();
    }

    /**
     * @param Carrier $carrierModel
     */
    public function save($carrierModel)
    {
        $this->resourceModel->save($carrierModel);
    }

    /**
     * Load carrier by id
     *
     * @param integer $id
     * @return Carrier
     */
    public function load($id)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Carrier $model */
        $model = $this->carrierFactory->create();
        $this->resourceModel->load($model, $id, null);
        return $model;
    }

    /**
     * Load carrier by code
     *
     * @param string $code
     * @return Carrier
     */
    public function loadByCode($code)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Carrier $model */
        $model = $this->carrierFactory->create();
        $this->resourceModel->load($model, $code, 'code');
        return $model;
    }

    /**
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Carrier $carrierModel
     */
    public function delete($carrierModel)
    {
        $this->resourceModel->delete($carrierModel);
    }
}
