<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Checkout\Api;

class GuestPaymentInformationPlugin
{
    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Quote\AddressManager
     */
    private $addressManager;

    /**
     * @var \Magento\Quote\Model\QuoteIdMaskFactory
     */
    private $quoteIdMaskFactory;

    /**
     * @var \Magento\Quote\Api\CartRepositoryInterface
     */
    private $quoteRepository;

    /**
     * GuestPaymentInformationPlugin constructor.
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Quote\AddressManager $addressManager
     * @param \Magento\Quote\Model\QuoteIdMaskFactory $quoteIdMaskFactory
     * @param \Magento\Quote\Api\CartRepositoryInterface $quoteRepository
     */
    public function __construct(
        \Bluebirdday\TranssmartSmartConnect\Model\Quote\AddressManager $addressManager,
        \Magento\Quote\Model\QuoteIdMaskFactory $quoteIdMaskFactory,
        \Magento\Quote\Api\CartRepositoryInterface $quoteRepository
    ) {
        $this->addressManager = $addressManager;
        $this->quoteIdMaskFactory = $quoteIdMaskFactory;
        $this->quoteRepository = $quoteRepository;
    }

    /**
     * @param \Magento\Checkout\Api\GuestPaymentInformationManagementInterface $subject
     * @param string $cartId
     * @param string $email
     * @param \Magento\Quote\Api\Data\PaymentInterface $paymentMethod
     * @param \Magento\Quote\Api\Data\AddressInterface|null $billingAddress
     * @return array
     */
    public function beforeSavePaymentInformationAndPlaceOrder(
        \Magento\Checkout\Api\GuestPaymentInformationManagementInterface $subject,
        $cartId,
        $email,
        \Magento\Quote\Api\Data\PaymentInterface $paymentMethod,
        \Magento\Quote\Api\Data\AddressInterface $billingAddress = null
    ) {
        /** @var $quoteIdMask \Magento\Quote\Model\QuoteIdMask */
        $quoteIdMask = $this->quoteIdMaskFactory->create()->load($cartId, 'masked_id');
        $quote = $this->quoteRepository->getActive($quoteIdMask->getQuoteId());
        $pickupAddress = $this->addressManager->getPickupAddress($quote);

        if ($pickupAddress && !$pickupAddress->getEmail()) {
            $pickupAddress->setEmail($email);
            $pickupAddress->save();
            // we need to have updated addresses in quote fetched from repository
            $this->quoteRepository->save($quote);
        }

        return [$cartId, $email, $paymentMethod, $billingAddress];
    }
}
