<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Checkout\Api;

class ShippingInformationPlugin
{
    /**
     * @var \Magento\Quote\Api\CartRepositoryInterface
     */
    private $quoteRepository;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Shipping\Method\Validator
     */
    private $methodValidator;

    /**
     * @var \Magento\Quote\Model\ResourceModel\Quote\Address\Rate
     */
    private $shippingRateResource;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Quote\AddressManager
     */
    private $addressManager;

    /**
     * ShippingInformationPlugin constructor.
     * @param \Magento\Quote\Api\CartRepositoryInterface $quoteRepository
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Shipping\Method\Validator $methodValidator
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Quote\AddressManager $addressManager
     * @param \Magento\Quote\Model\ResourceModel\Quote\Address\Rate $shippingRateResource
     */
    public function __construct(
        \Magento\Quote\Api\CartRepositoryInterface $quoteRepository,
        \Bluebirdday\TranssmartSmartConnect\Model\Shipping\Method\Validator $methodValidator,
        \Bluebirdday\TranssmartSmartConnect\Model\Quote\AddressManager $addressManager,
        \Magento\Quote\Model\ResourceModel\Quote\Address\Rate $shippingRateResource
    ) {
        $this->addressManager = $addressManager;
        $this->quoteRepository = $quoteRepository;
        $this->methodValidator = $methodValidator;
        $this->shippingRateResource = $shippingRateResource;
    }

    public function beforeSaveAddressInformation(
        \Magento\Checkout\Api\ShippingInformationManagementInterface $subject,
        $cartId,
        \Magento\Checkout\Api\Data\ShippingInformationInterface $addressInformation
    ) {
        if ($addressInformation->getShippingCarrierCode() == 'transsmartpickup') {
            $this->methodValidator->validate($addressInformation->getShippingMethodCode());
        }

        $quote = $this->quoteRepository->getActive($cartId);
        // we need to update pickup address only
        if ($addressInformation->getExtensionAttributes()
            && $addressInformation->getExtensionAttributes()->getPickupAddress()
        ) {
            $pickupAddress = $addressInformation->getExtensionAttributes()->getPickupAddress();
            $street = $pickupAddress->getStreet();
            if (is_array($street) && count($street) > 1) {
                $pickupAddress->setStreet(implode(' ', $street));
            }

            // we will reuse this data on further shipping calculations
            $methodParts = [
                $addressInformation->getShippingCarrierCode(),
                $addressInformation->getShippingMethodCode()
            ];
            $shippingMethod = implode('_', $methodParts);
            $pickupAddress->setShippingMethod($shippingMethod);
            if ($addressInformation->getExtensionAttributes()->getPickupLocationId()) {
                $pickupAddress->setCompany(implode(
                    '::',
                    [
                        $pickupAddress->getCompany(),
                        $addressInformation->getExtensionAttributes()->getPickupLocationId()
                    ]
                ));
            }

            //when pickup address is already saved, we update its data
            $savedAddress = $quote->getExtensionAttributes()->getPickupAddress();
            if ($savedAddress) {
                $newData = $pickupAddress->getData();
                if ($pickupAddress->getId()) {
                    unset($newData['id']);
                }
                // copy update saved address data
                foreach ($newData as $name => $value) {
                    $savedAddress->setData($name, $value);
                }

                $savedAddress->save();

            } else {
                $pickupAddress->setAddressType('pickup');
                $pickupAddress->setQuoteId($cartId);
                $pickupAddress->setShippingMethod($shippingMethod);
                $pickupAddress->setEmail($quote->getShippingAddress()->getEmail());
                $pickupAddress->save();
            }
            //reset repository cache for this quote
            $this->quoteRepository->save($quote);
        } else {
            $pickupAddress = $this->addressManager->getPickupAddress($quote);
            if ($pickupAddress) {
                $pickupAddress->isDeleted(true);
                $pickupAddress->save();
                $this->quoteRepository->save($quote);
            }
        }

        return [$cartId, $addressInformation];
    }
}
