<?php
namespace Bluebirdday\TranssmartSmartConnect\Model;

use Magento\Checkout\Model\ConfigProviderInterface;

/**
 * Provide carrier images for precaching on checkout
 */
class CheckoutConfigProvider implements ConfigProviderInterface
{
    /**
     * @var \Magento\Framework\Filesystem
     */
    private $filesystem;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Media\Config
     */
    private $mediaConfig;

    /**
     * CheckoutConfigProvider constructor.
     * @param \Magento\Framework\Filesystem $filesystem
     * @param Media\Config $mediaConfig
     */
    public function __construct(
        \Magento\Framework\Filesystem $filesystem,
        \Bluebirdday\TranssmartSmartConnect\Model\Media\Config $mediaConfig
    ) {
        $this->filesystem = $filesystem;
        $this->mediaConfig = $mediaConfig;
    }

    /**
     * @param $fileName
     * @return bool
     */
    private function isImage($fileName)
    {
        $images = ['jpg', 'png', 'jpeg', 'gif', 'bmp', 'JPG', 'JPEG', 'PNG', 'GIF'];

        $nameParts = explode('.', $fileName);
        $lastPart = array_pop($nameParts);
        return in_array($lastPart, $images);
    }

    /**
     * {@inheritdoc}
     */
    public function getConfig()
    {
        $config = [];
        $mediaDirectory = $this->filesystem->getDirectoryWrite(
            \Magento\Framework\App\Filesystem\DirectoryList::MEDIA
        );
        $relativePath = $this->mediaConfig->getBaseMediaPath();
        $absolutePath = $mediaDirectory->getAbsolutePath($relativePath);
        $files = glob($absolutePath . '/*');

        $urls = [];
        foreach ($files as $file) {
            if (is_file($file) && is_readable($file) && $this->isImage($file)) {
                $urls[] = $this->mediaConfig->getMediaUrl(basename($file));
            }
        }

        $config['pickup_locations']['images'] = $urls;
        return $config;
    }
}
