<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Config\Config\Structure\Element;

use \Magento\Config\Model\Config\Structure\Element\Section as OriginalSection;
use Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile\AllowedCountry;
use Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileRepository;

class SectionPlugin
{
    const CONFIG_TRANSSMART_PROFILES_SECTION_ID = 'transsmart_profiles';

    /**
     * @var ProfileRepository
     */
    private $profileRepository;

    /**
     * SectionPlugin constructor.
     * @param ProfileRepository $profileRepository
     */
    public function __construct(
        ProfileRepository $profileRepository
    ) {
        $this->profileRepository = $profileRepository;
    }

    /**
     * @param OriginalSection $subject
     * @param array $data
     * @param string $scope
     * @return array
     */
    public function beforeSetData(OriginalSection $subject, array $data, $scope) {
        if($data['id'] == AllowedCountry::CONFIG_TRANSSMART_PROFILES_SECTION_ID) {
            $dynamicGroups = $this->getDynamicConfigGroups();

            if(!empty($dynamicGroups)) {
                if (array_key_exists('children', $data)) {
                    $data['children'] += $dynamicGroups;
                } else {
                    $data['children'] = $dynamicGroups;
                }
            }
        }

        return [$data, $scope];
    }

    /**
     * @return array
     */
    private function getDynamicConfigGroups()
    {
        $profiles = $this->profileRepository->getList();
        $dynamicConfigGroups = [];

        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile $profile */
        foreach ($profiles as $index => $profile) {
            $dynamicConfigFields = [];
            $path = implode('/', [AllowedCountry::CONFIG_TRANSSMART_PROFILES_SECTION_ID, $profile->getCode()]);

            $dynamicConfigFields['sallowspecific'] = [
                'id' => 'sallowspecific',
                'translate' => '',
                'type' => 'select',
                'sortOrder' => '10',
                'showInDefault' => '1',
                'showInWebsite' => '1',
                'showInStore' => '0',
                'label' => __('Ship to Applicable Countries'),
                'frontend_class' => 'shipping-applicable-country',
                'source_model' => 'Magento\\Shipping\\Model\\Config\\Source\\Allspecificcountries',
                '_elementType' => 'field',
                'path' => $path,
            ];

            $dynamicConfigFields['specificcountry'] = [
                'id' => 'specificcountry',
                'translate' => '',
                'type' => 'multiselect',
                'sortOrder' => '20',
                'showInDefault' => '1',
                'showInWebsite' => '1',
                'showInStore' => '0',
                'label' => __('Ship to Specific Countries'),
                'can_be_empty' => '1',
                'source_model' => 'Magento\\Directory\\Model\\Config\\Source\\Country',
                '_elementType' => 'field',
                'path' => $path,
            ];

            $dynamicConfigGroups[$profile->getCode()] = [
                'id' => 'profile-configuration-' . $profile->getCode(),
                'translate' => 'label comment',
                'type' => 'text',
                'sortOrder' => $index * 10,
                'showInDefault' => '1',
                'showInWebsite' => '0',
                'showInStore' => '0',
                'comment' => '',
                'label' => __('[%1] %2 Configuration', $profile->getCode(), $profile->getDescription()),
                '_elementType' => 'group',
                'path' => AllowedCountry::CONFIG_TRANSSMART_PROFILES_SECTION_ID,
                'children' => $dynamicConfigFields,
            ];
        }

        return $dynamicConfigGroups;
    }
}
