<?php
namespace Bluebirdday\TranssmartSmartConnect\Model;

/**
 * Class CostcenterRepository
 * @package Bluebirdday\TranssmartSmartConnect\Model
 */
class CostcenterRepository implements \Bluebirdday\TranssmartSmartConnect\Model\Entity\RepositoryInterface
{
    /**
     * @var CostcenterFactory
     */
    protected $costcenterFactory;

    /**
     * @var Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Costcenter\CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Package
     */
    protected $resourceModel;

    /**
     * CostcenterRepository constructor.
     * @param CostcenterFactory $costcenterFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Costcenter\CollectionFactory $collectionFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Costcenter $resourceModel
     */
    public function __construct(
        \Bluebirdday\TranssmartSmartConnect\Model\CostcenterFactory $costcenterFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Costcenter\CollectionFactory $collectionFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Costcenter $resourceModel
    ) {
        $this->costcenterFactory = $costcenterFactory;
        $this->collectionFactory = $collectionFactory;
        $this->resourceModel = $resourceModel;
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Costcenter[]
     */
    public function getList()
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Costcenter\Collection $collection */
        $collection = $this->collectionFactory->create();
        return $collection->getItems();
    }

    /**
     * @param Costcenter $costcenterModel
     */
    public function save($costcenterModel)
    {
        $this->resourceModel->save($costcenterModel);
    }

    /**
     * Load costcenter by id
     *
     * @param integer $id
     * @return Costcenter
     */
    public function load($id)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Costcenter $model */
        $model = $this->costcenterFactory->create();
        $this->resourceModel->load($model, $id, null);
        return $model;
    }

    /**
     * Load costcenter by code
     *
     * @param string $code
     * @return Costcenter
     */
    public function loadByCode($code)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Costcenter $model */
        $model = $this->costcenterFactory->create();
        $this->resourceModel->load($model, $code, 'code');
        return $model;
    }

    /**
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Costcenter $costcenterModel
     */
    public function delete($costcenterModel)
    {
        $this->resourceModel->delete($costcenterModel);
    }
}
