<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Data;

/**
 * Data container for address used in requests
 */
class Address extends \Bluebirdday\TranssmartSmartConnect\Model\Data\Container
{
    /**
     * @var array
     */
    protected $requiredFields = [
        'type',
        'name',
        'addressLine1',
        'houseNo',
        'city',
        'zipCode',
        'country',
    ];

    /**
     * @var array
     */
    protected $optionalFields = [
        'addressLine2',
        'addressLine3',
        'state',
        'contact',
        'telNo',
        'faxNo',
        'email',
        'accountNumber',
        'customerNumber',
        'vatNumber',
        'residential'
    ];

    /**
     * Get address type
     * "RECV" or "SEND" for rate requests
     * "INVC" for invoice address on booking request
     * "3PTY" for third party billing
     *
     * @return string
     */
    public function getType()
    {
        return $this->getData('type');
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->getData('name');
    }

    /**
     * @return string
     */
    public function getAddressLine1()
    {
        return $this->getData('addressLine1');
    }

    /**
     * Get street
     *
     * @return string
     */
    public function getAddressLine2()
    {
        return $this->getData('addressLine2');
    }

    /**
     * @return string|null
     */
    public function getAddressLine3()
    {
        return $this->getData('addressLine3');
    }

    /**
     * Get house number including extension letters
     *
     * @return string
     */
    public function getHouseNo()
    {
        return $this->getData('houseNo');
    }

    /**
     * Get city
     *
     * @return string
     */
    public function getCity()
    {
        return $this->getData('city');
    }

    /**
     * Get ZIP code
     *
     * @return string
     */
    public function getZipCode()
    {
        return $this->getData('zipCode');
    }

    /**
     * Get state
     *
     * @return string|null
     */
    public function getState()
    {
        return $this->getData('state');
    }

    /**
     * Get two-letter county code
     *
     * @return string
     */
    public function getCountry()
    {
        return $this->getData('country');
    }

    /**
     * Get contact name for organizations
     *
     * @return string|null
     */
    public function getContact()
    {
        return $this->getData('contact');
    }

    /**
     * @return string|null
     */
    public function getTelNo()
    {
        return $this->getData('telNo');
    }

    /**
     * @return string|null
     */
    public function getFaxNo()
    {
        return $this->getData('faxNo');
    }

    /**
     * @return string|null
     */
    public function getEmail()
    {
        return $this->getData('email');
    }

    /**
     * @return string|null
     */
    public function getAccountNumber()
    {
        return $this->getData('accountNumber');
    }

    /**
     * @return string|null
     */
    public function getCustomerNumber()
    {
        return $this->getData('customerNumber');
    }

    /**
     * @return string|null
     */
    public function getVatNumber()
    {
        return $this->getData('vatNumber');
    }

    /**
     * Get address residential status
     *
     * 0 - company address
     * 1 - home address
     * @return int|null
     */
    public function getResidential()
    {
        return $this->getData('residential');
    }
}
