<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Data;

/**
 * Parent class for data containers with common methods like toArray() and __toString()
 *
 * @package Bluebirdday\TranssmartSmartConnect\Model\Data
 */
class Container extends \Magento\Framework\DataObject
{
    protected $requiredFields = [];

    protected $optionalFields = [];

    /**
     * Container constructor.
     * @param array $data
     * @param array $requiredFields
     * @param array $optionalFields
     */
    public function __construct(
        array $data = [],
        array $requiredFields = [],
        array $optionalFields = []
    ) {
        parent::__construct($data);

        if (!empty($requiredFields)) {
            $this->requiredFields = $requiredFields;
        }

        if (!empty($optionalFields)) {
            $this->optionalFields = $optionalFields;
        }
    }

    /**
     * Make sure that all the required fields are set
     *
     * @throws \RuntimeException
     */
    protected function checkRequiredFields()
    {
        foreach ($this->requiredFields as $fieldName) {
            if (null === $this->getData($fieldName)) {
                throw new \RuntimeException("Field '$fieldName' could not be empty");
            }
        }
    }

    /**
     * Prepare data for URL output for GET requests
     *
     * @return string
     */
    public function __toString()
    {
        $variables = $this->toArray();

        return http_build_query($variables);
    }

    /**
     * Prepare data as array for POST requests
     *
     * @param array $keys
     * @return array
     */
    public function toArray(array $keys = [])
    {
        $variables = [];
        $this->checkRequiredFields();

        $fields = array_merge($this->optionalFields, $this->requiredFields);
        foreach ($fields as $field) {
            $value = $this->getData($field);
            if ($value !== null) {
                $variables[$field] =  $this->getData($field);
            }
        }

        return $variables;
    }
}
