<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Data;

/**
 * Data container for shipping rate request
 */
class Rate extends \Bluebirdday\TranssmartSmartConnect\Model\Data\Container
{
    protected $requiredFields = [
        'reference',
        'measurements',
        'addresses',
        'packages',
    ];

    protected $optionalFields = [
        'carrier',
        'costcenter',
        'mailType',
        'language',
        'selectedCarrier',
        'executingCarrier',
        'airwayBillNumber',
        'trackingAndTraceUrl',
        'description',
        'instruction',
        'value',
        'valueCurrency',
        'currency',
        'spotPrice',
        'spotPriceCurrency',
        'price',
        'salesPrice',
        'shipmentStatusCode',
        'pickupDate',
        'pickupTime',
        'pickupTimeTo',
        'requestedDeliveryDate',
        'requestedDeliveryTime',
        'requestedDeliveryTimeTo',
        'creationDate',
        'service',
        'serviceLevelTime',
        'serviceLevelOther',
        'incoterms',
        'inbound',
        'loadmeters',
        'numberOfPackages',
        'errorCode',
        'errorDescription',
        'bookedDate',
//        'deliveryNoteInformation',
//        'additionalReferences',
    ];

    public function toArray(array $keys = [])
    {
        $variables = [];
        $this->checkRequiredFields();

        $requiredFields = $this->requiredFields;
        if ($this->getMeasurements() instanceof  \Bluebirdday\TranssmartSmartConnect\Model\Data\Measurements) {
            $variables['measurements'] = $this->getMeasurements()->toArray();
            unset($requiredFields[array_search('measurements', $requiredFields)]);
        }

        foreach ($this->getAddresses() as $address) {
            $variables['addresses'][] = $address->toArray();
        }
        unset($requiredFields[array_search('addresses', $requiredFields)]);

        foreach ($this->getPackages() as $package) {
            $variables['packages'][] = $package->toArray();
        }
        unset($requiredFields[array_search('packages', $requiredFields)]);

        //convert scalar fields
        $fields = array_merge($this->optionalFields, $requiredFields);
        foreach ($fields as $field) {
            if (null === $this->getData($field)) {
                $variables[$field] = '';
            } else {
                $variables[$field] =  $this->getData($field);
            }
        }

        return $variables;
    }

    public function getReference()
    {
        return $this->getData('reference');
    }

    public function getCarrier()
    {
        return $this->getData('carrier');
    }

    public function getCostcenter()
    {
        return $this->getData('costcenter');
    }

    public function getMailType()
    {
        return $this->getData('mailType');
    }

    public function getLanguage()
    {
        return $this->getData('language');
    }

    public function getSelectedCarrier()
    {
        return $this->getData('selectedCarrier');
    }

    public function getExecutingCarrier()
    {
        return $this->getData('executingCarrier');
    }

    public function getAirwayBillNumber()
    {
        return $this->getData('airwayBillNumber');
    }

    public function getTrackingAndTraceUrl()
    {
        return $this->getData('trackingAndTraceUrl');
    }

    public function getDescription()
    {
        return $this->getData('description');
    }

    public function getInstruction()
    {
        return $this->getData('instruction');
    }

    public function getValue()
    {
        return $this->getData('value');
    }

    public function getValueCurrency()
    {
        return $this->getData('valueCurrency');
    }

    public function getCurrency()
    {
        return $this->getData('currency');
    }

    public function getSpotPrice()
    {
        return $this->getData('spotPrice');
    }

    public function getSpotPriceCurrency()
    {
        return $this->getData('spotPriceCurrency');
    }

    public function getPrice()
    {
        return $this->getData('price');
    }

    public function getSalesPrice()
    {
        return $this->getData('salesPrice');
    }

    public function getShipmentStatusCode()
    {
        return $this->getData('shipmentStatusCode');
    }

    public function getPickupDate()
    {
        return $this->getData('pickupDate');
    }

    public function getPickupTime()
    {
        return $this->getData('pickupTime');
    }

    public function getPickupTimeTo()
    {
        return $this->getData('pickupTimeTo');
    }

    public function getRequestedDeliveryDate()
    {
        return $this->getData('requestedDeliveryDate');
    }

    public function getRequestedDeliveryTime()
    {
        return $this->getData('requestedDeliveryTime');
    }

    public function getRequestedDeliveryTimeTo()
    {
        return $this->getData('requestedDeliveryTimeTo');
    }

    public function getCreationDate()
    {
        return $this->getData('creationDate');
    }

    public function getService()
    {
        return $this->getData('service');
    }

    public function getServiceLevelTime()
    {
        return $this->getData('serviceLevelTime');
    }

    public function getServiceLevelOther()
    {
        return $this->getData('serviceLevelOther');
    }

    public function getIncoterms()
    {
        return $this->getData('incoterms');
    }

    public function getInbound()
    {
        return $this->getData('inbound');
    }

    public function getLoadmeters()
    {
        return $this->getData('loadmeters');
    }

    public function getNumberOfPackages()
    {
        return $this->getData('numberOfPackages');
    }

    public function getErrorCode()
    {
        return $this->getData('errorCode');
    }

    public function getErrorDescription()
    {
        return $this->getData('errorDescription');
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Data\Measurements|null
     */
    public function getMeasurements()
    {
        return $this->getData('measurements');
    }

    public function getBookedDate()
    {
        return $this->getData('bookedDate');
    }

    public function getDeliveryNoteInformation()
    {
        return $this->getData('deliveryNoteInformation');
    }

    public function getAdditionalReferences()
    {
        return $this->getData('additionalReferences');
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Data\Address[]
     */
    public function getAddresses()
    {
        return $this->getData('addresses');
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Data\Package[]
     */
    public function getPackages()
    {
        return $this->getData('packages');
    }
}
