<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Entity\Updater;

use Magento\Framework\Exception\LocalizedException;

class Pool
{
    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Entity\UpdaterInterface[]
     */
    protected $updaters;

    /**
     * Pool constructor.
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Entity\UpdaterInterface[] $updaters
     */
    public function __construct(
        array $updaters
    ) {
        $this->updaters = $updaters;
    }

    /**
     * Common method to get entity updates
     *
     * @throws LocalizedException
     */
    public function refresh()
    {
        foreach ($this->updaters as $updater) {
            if (method_exists($updater, 'execute')) {
                call_user_func([$updater, 'execute']);
            } else {
                throw new LocalizedException(__('Unexpected updater "%1"', get_class($updater)));
            }
        }
    }
}
