<?php
namespace Bluebirdday\TranssmartSmartConnect\Model;

use Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Incoterm\CollectionFactory as IncotermCollectionFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Psr\Log\LoggerInterface;

/**
 * Class IncotermManagement
 * @package Bluebirdday\TranssmartSmartConnect\Model
 */
class IncotermManagement
{
    const CONFIG_DEFAULT_PATH = 'transsmart/default_shipment_properties/incoterm';

    /**
     * @var IncotermCollectionFactory
     */
    private $collectionFactory;

    /**
     * @var IncotermRepository
     */
    private $incotermRepository;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * IncotermManagement constructor.
     * @param IncotermCollectionFactory $collectionFactory
     * @param IncotermRepository $incotermRepository
     * @param ScopeConfigInterface $scopeConfig
     * @param LoggerInterface $logger
     */
    public function __construct(
        IncotermCollectionFactory $collectionFactory,
        IncotermRepository $incotermRepository,
        ScopeConfigInterface $scopeConfig,
        LoggerInterface $logger
    ) {
        $this->collectionFactory = $collectionFactory;
        $this->incotermRepository = $incotermRepository;
        $this->scopeConfig = $scopeConfig;
        $this->logger = $logger;
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Incoterm|null
     */
    public function getDefault()
    {
        $configDefault = $this->scopeConfig->getValue(static::CONFIG_DEFAULT_PATH, 'website');
        if ($configDefault === null) {
            $this->logger->error('Default incoterm is not configured. Please, re-save config.');
        } else {
            $defaultIncoterm = $this->incotermRepository->load($configDefault);
            if ($defaultIncoterm->getId() == $configDefault) {
                return $defaultIncoterm;
            } else {
                $this->logger->error('Wrong default incoterm: ' . $configDefault);
            }
        }

        // if default is not configured or is deprecated, we fall back to Transsmart default
        $transsmartDefault = $this->getTranssmartDefault();
        if ($transsmartDefault) {
            return $transsmartDefault;
        }
        // panic mode: try to return first incoterm in the table if any
        return $this->getFirst();
    }

    /**
     * @return null|Incoterm
     */
    private function getTranssmartDefault()
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Incoterm\Collection $collection */
        $collection = $this->collectionFactory->create();
        $collection->addFieldToFilter('is_default', 1);
        if ($collection->count() === 0) {
            return null;
        }
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Incoterm $firstItem */
        $firstItem = $collection->getFirstItem();
        return $firstItem;
    }

    private function getFirst()
    {
        $collection = $this->collectionFactory->create();
        $collection->getSelect()->limit(1);
        if ($collection->count() === 0) {
            return null;
        }
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Incoterm $firstItem */
        $firstItem = $collection->getFirstItem();
        return $firstItem;
    }
}
