<?php
namespace Bluebirdday\TranssmartSmartConnect\Model;
use Magento\Framework\Exception\CouldNotSaveException;

/**
 * Class IncotermRepository
 * @package Bluebirdday\TranssmartSmartConnect\Model
 */
class IncotermRepository implements \Bluebirdday\TranssmartSmartConnect\Model\Entity\RepositoryInterface
{
    /**
     * @var IncotermFactory
     */
    protected $incotermFactory;

    /**
     * @var Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Incoterm\CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Package
     */
    protected $resourceModel;

    /**
     * IncotermRepository constructor.
     * @param IncotermFactory $incotermFactory
     * @param Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Incoterm\CollectionFactory $collectionFactory
     * @param Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Incoterm $resourceModel
     */
    public function __construct(
        \Bluebirdday\TranssmartSmartConnect\Model\IncotermFactory $incotermFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Incoterm\CollectionFactory $collectionFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Incoterm $resourceModel
    ) {
        $this->incotermFactory = $incotermFactory;
        $this->collectionFactory = $collectionFactory;
        $this->resourceModel = $resourceModel;
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Incoterm[]
     */
    public function getList()
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Incoterm\Collection $collection */
        $collection = $this->collectionFactory->create();
        return $collection->getItems();
    }

    /**
     * @param Incoterm $incotermModel
     */
    public function save($incotermModel)
    {
        $this->resourceModel->save($incotermModel);
    }

    /**
     * Load incoterm by id
     *
     * @param integer $id
     * @return Incoterm
     */
    public function load($id)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Incoterm $model */
        $model = $this->incotermFactory->create();
        $this->resourceModel->load($model, $id, null);
        return $model;
    }


    /**
     * Load incoterm by code
     *
     * @param string $code
     * @return Incoterm
     * @throws CouldNotSaveException
     */
    public function loadByCode($code)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Incoterm $model */
        $model = $this->incotermFactory->create();
        $this->resourceModel->load($model, $code, 'code');

        if (!$model->getId()) {
            throw new CouldNotSaveException(__('Cannot load incoterm with code %1', $code));
        }

        return $model;
    }

    /**
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Incoterm $incotermModel
     */
    public function delete($incotermModel)
    {
        $this->resourceModel->delete($incotermModel);
    }
}
