<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Location;

/**
 * Location Request Data Container
 *
 * @package Bluebirdday\TranssmartSmartConnect\Model
 */
class Request extends \Bluebirdday\TranssmartSmartConnect\Model\Data\Container
{
    protected $requiredFields = [
        'city' => 'city',
        'zip_code' => 'zipCode',
        'street' => 'street',
        'house_number' => 'houseNumber',
        'country_from'=> 'countryFrom',
        'country_to' => 'countryTo',
        'provider' => 'provider',
        'limit' => 'limit',
    ];

    protected $optionalFields = [
        'house_number_ext' => 'houseNumberExt',
    ];

    /**
     * @return string
     */
    public function getStreet()
    {
        return $this->getData('street');
    }

    /**
     * @param string $value
     */
    public function setStreet($value)
    {
        $this->setData('street', $value);
    }

    /**
     * @return string|int
     */
    public function getHouseNumber()
    {
        return $this->getData('house_number');
    }

    /**
     * @param string|int $value
     */
    public function setHouseNumber($value)
    {
        $this->setData('house_number', $value);
    }

    /**
     * @return string
     */
    public function getHouseNumberExt()
    {
        return $this->getData('house_number_ext');
    }

    /**
     * @param string $value
     */
    public function setHouseNumberExt($value)
    {
        $this->setData('house_number_ext', $value);
    }

    /**
     * @return string
     */
    public function getZipCode()
    {
        return $this->getData('zip_code');
    }

    /**
     * @param string $value
     */
    public function setZipCode($value)
    {
        $this->getData('zip_code', $value);
    }

    /**
     * @return string
     */
    public function getCity()
    {
        return $this->getData('city');
    }

    /**
     * @param $value
     */
    public function setCity($value)
    {
        $this->getData('city', $value);
    }

    /**
     * Get two-letter origin country code
     *
     * @return string
     */
    public function getCountryFrom()
    {
        return $this->getData('country_from');
    }

    /**
     * @param $value
     */
    public function setCountryFrom($value)
    {
        $this->getData('country_from', $value);
    }

    /**
     * Get two-letter destination country code
     *
     * @return string
     */
    public function getCountryTo()
    {
        return $this->getData('country_to');
    }

    /**
     * @param $value
     */
    public function setCountryTo($value)
    {
        $this->getData('country_to', $value);
    }

    /**
     * Shipping provider(carrier) code or "ALL"
     *
     * @return string
     */
    public function getProvider()
    {
        return $this->getData('provider');
    }

    /**
     * @param $value
     */
    public function setProvider($value)
    {
        $this->setData('provider', $value);
    }

    /**
     * @return int|string
     */
    public function getLimit()
    {
        return $this->getData('limit');
    }

    public function setLimit($value)
    {
        $this->setData('limit', $value);
    }

    /**
     * {@inheritdoc}
     */
    protected function checkRequiredFields()
    {
        foreach ($this->requiredFields as $internal => $external) {
            if (null === $this->getData($internal)) {
                throw new \RuntimeException("Field '$internal' could not be empty");
            }
        }
    }

    /**
     * Prepare data for URL output
     *
     * @return string
     */
    public function __toString()
    {
        $variables = [];
        $this->checkRequiredFields();
        $fields = $this->requiredFields + $this->optionalFields;

        foreach ($fields as $internal => $external) {
            $variables[$external] = $this->getData($internal);
        }

        return http_build_query($variables);
    }
}
