<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Mail;

use Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Mail\Type\CollectionFactory as MailtypeCollectionFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Psr\Log\LoggerInterface;

class TypeManagement
{
    const CONFIG_DEFAULT_PATH = 'transsmart/default_shipment_properties/mail_type';

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Mail\Type\CollectionFactory
     */
    private $collectionFactory;

    /**
     * @var TypeRepository
     */
    private $mailtypeRepository;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @param MailtypeCollectionFactory $collectionFactory
     * @param TypeRepository $mailtypeRepository
     * @param ScopeConfigInterface $scopeConfig
     * @param LoggerInterface $logger
     */
    public function __construct(
        MailtypeCollectionFactory $collectionFactory,
        TypeRepository $mailtypeRepository,
        ScopeConfigInterface $scopeConfig,
        LoggerInterface $logger
    ) {
        $this->collectionFactory = $collectionFactory;
        $this->mailtypeRepository = $mailtypeRepository;
        $this->scopeConfig = $scopeConfig;
        $this->logger = $logger;
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Mail\Type|null
     */
    public function getDefault()
    {
        $configDefault = $this->scopeConfig->getValue(static::CONFIG_DEFAULT_PATH, 'website');

        if ($configDefault === null) {
            $this->logger->error('Default mail type is not configured. Please, re-save config.');
        } else {
            $defaultMailType = $this->mailtypeRepository->load($configDefault);
            if ($defaultMailType->getId() == $configDefault) {
                return $defaultMailType;
            } else {
                $this->logger->error('Wrong default mailtype: ' . $configDefault);
            }
        }

        // if default is not configured or is deprecated, we fall back to Transsmart default
        $transsmartDefault = $this->getTranssmartDefault();
        if ($transsmartDefault) {
            return $transsmartDefault;
        }
        // panic mode: try to return first mail type in the table if any
        return $this->getFirst();
    }

    /**
     * @return null|Type
     */
    private function getTranssmartDefault()
    {
        $collection = $this->collectionFactory->create();
        $collection->addFieldToFilter('is_default', 1);
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Mail\Type $firstItem */
        if ($collection->count() === 0) {
            return null;
        }
        $firstItem = $collection->getFirstItem();
        return $firstItem;
    }

    /**
     * @return null|Type
     */
    private function getFirst()
    {
        $collection = $this->collectionFactory->create();
        $collection->getSelect()->limit(1);
        if ($collection->count() === 0) {
            return null;
        }
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Mail\Type $firstItem */
        $firstItem = $collection->getFirstItem();
        return $firstItem;
    }
}
