<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Mail;

/**
 * Class TypeRepository
 * Basic operations with mail types.
 */
class TypeRepository implements \Bluebirdday\TranssmartSmartConnect\Model\Entity\RepositoryInterface
{
    /**
     * @var TypeFactory
     */
    protected $typeFactory;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Mail\Type\Collection
     */
    protected $collectionFactory;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Mail\Type
     */
    protected $resourceModel;

    /**
     * TypeRepository constructor.
     * @param TypeFactory $typeFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Mail\Type\CollectionFactory $collectionFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Mail\Type $resourceModel
     */
    public function __construct(
        \Bluebirdday\TranssmartSmartConnect\Model\Mail\TypeFactory $typeFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Mail\Type\CollectionFactory $collectionFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Mail\Type $resourceModel
    ) {
        $this->typeFactory = $typeFactory;
        $this->collectionFactory = $collectionFactory;
        $this->resourceModel = $resourceModel;
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Mail\Type[]
     */
    public function getList()
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Mail\Type\Collection $collection */
        $collection = $this->collectionFactory->create();
        return $collection->getItems();
    }

    /**
     * @throws \Exception
     * @throws \Magento\Framework\Exception\AlreadyExistsException
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Mail\Type $typeModel
     */
    public function save($typeModel)
    {
        $this->resourceModel->save($typeModel);
    }

    /**
     * @param integer $id
     * @return Type
     */
    public function load($id)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Mail\Type $model */
        $model = $this->typeFactory->create();
        $this->resourceModel->load($model, $id, null);
        return $model;
    }

    /**
     * @param string $code
     * @return Type
     */
    public function loadByCode($code)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Mail\Type $model */
        $model = $this->typeFactory->create();
        $this->resourceModel->load($model, $code, 'code');
        return $model;
    }

    /**
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Mail\Type $typeModel
     */
    public function delete($typeModel)
    {
        $this->resourceModel->delete($typeModel);
    }
}
