<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Media;

use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Config
 *
 * Get URLs and paths for SmartConnect media files
 *
 * @package Bluebirdday\TranssmartSmartConnect\Model\Media
 */
class Config
{
    /**
     * Store manager
     *
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(StoreManagerInterface $storeManager)
    {
        $this->storeManager = $storeManager;
    }

    /**
     * Get relative path of SmartConnect media files
     *
     * @return string
     */
    public function getBaseMediaPath()
    {
        return 'transsmart/smartconnect';
    }

    /**
     * Get URL of SmartConnect media dir
     *
     * @return string
     */
    public function getBaseMediaUrl()
    {
        return $this->storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA)
            . $this->getBaseMediaPath();
    }

    /**
     * @param string $file
     * @return string
     */
    public function getMediaUrl($file)
    {
        return $this->getBaseMediaUrl() . '/' . $this->_prepareFile($file);
    }

    /**
     * Get relative path of media file
     *
     * @param string $file
     * @return string
     */
    public function getMediaPath($file)
    {
        return $this->getBaseMediaPath() . '/' . $this->_prepareFile($file);
    }

    /**
     * Help windows fans to survive
     *
     * @param string $file
     * @return string
     */
    protected function _prepareFile($file)
    {
        return ltrim(str_replace('\\', '/', $file), '/');
    }
}
