<?php
namespace Bluebirdday\TranssmartSmartConnect\Model;

/**
 * Class Package
 * SmartConnect Package Model
 */
class Package extends \Magento\Framework\Model\AbstractExtensibleModel
{
    /**
     * {@inheritdoc}
     */
    protected function _construct()
    {
        $this->_init(\Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Package::class);
    }

    /**
     * Get package code
     *
     * @return string
     */
    public function getCode()
    {
        return $this->getData('code');
    }

    /**
     * Set package code
     *
     * @param string $value
     */
    public function setCode($value)
    {
        $this->setData('code', $value);
    }

    /**
     * Get package description
     *
     * @return string|null
     */
    public function getDescription()
    {
        return $this->getData('description');
    }

    /**
     * Set package description
     *
     * @param string|null $value
     */
    public function setDescription($value)
    {
        $this->setData('description', $value);
    }

    /**
     * Is default package for SmartConnect
     *
     * @return bool
     */
    public function getIsDefault()
    {
        return (bool)$this->getData('is_default');
    }

    /**
     * Set 'is default' package status
     *
     * @param bool $value
     */
    public function setIsDefault($value)
    {
        $this->setData('is_default', (bool)$value);
    }

    /**
     * Get package type
     *
     * @return string
     */
    public function getType()
    {
        return $this->getData('type');
    }

    /**
     * Set package type
     *
     * @param string $value
     */
    public function setType($value)
    {
        $this->setData('type', $value);
    }

    /**
     * Get linear unit of measurement
     *
     * @return string
     */
    public function getLinearUom()
    {
        return $this->getData('linear_uom');
    }

    /**
     * Set linear unit of measurement
     *
     * @param string $value
     */
    public function setLinearUom($value)
    {
        $this->setData('linear_uom', $value);
    }

    /**
     * Get mass unit of measurement
     *
     * @return string
     */
    public function getMassUom()
    {
        return $this->getData('mass_uom');
    }

    /**
     * Set mass unit of measurement
     *
     * @param string $value
     */
    public function setMassUom($value)
    {
        $this->setData('mass_uom', $value);
    }

    /**
     * Get package length
     *
     * @return string
     */
    public function getLength()
    {
        return $this->getData('length');
    }

    /**
     * Set package length
     *
     * @param string $value
     */
    public function setLength($value)
    {
        $this->setData('length', $value);
    }

    /**
     * Get package width
     *
     * @return string
     */
    public function getWidth()
    {
        return $this->getData('width');
    }

    /**
     * Set package width
     *
     * @param $value
     */
    public function setWidth($value)
    {
        $this->setData('width', $value);
    }

    /**
     * Get package height
     *
     * @return string
     */
    public function getHeight()
    {
        return $this->getData('height');
    }

    /**
     * Set package height
     *
     * @param string $value
     */
    public function setHeight($value)
    {
        $this->setData('height', $value);
    }

    /**
     * Get package weight
     *
     * @return string
     */
    public function getWeight()
    {
        return $this->getData('weight');
    }

    /**
     * Set package weight
     *
     * @param string $value
     */
    public function setWeight($value)
    {
        $this->setData('weight', $value);
    }
}
