<?php
namespace Bluebirdday\TranssmartSmartConnect\Model;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Package\CollectionFactory;

/**
 * Additional functions for Packages out of repository
 */
class PackageManagement
{
    const DEFAULT_PACKAGE_PATH = 'transsmart/default_shipment_properties/package_type';

    /**
     * @var Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Package\CollectionFactory
     */
    private $collectionFactory;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @var PackageRepository
     */
    private $packageRepository;

    /**
     * PackageManagement constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param CollectionFactory $collectionFactory
     * @param PackageRepository $packageRepository
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        CollectionFactory $collectionFactory,
        PackageRepository $packageRepository
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->collectionFactory = $collectionFactory;
        $this->packageRepository = $packageRepository;
    }

    /**
     * Get default package if any
     *
     * First we look at configuration saved, if no data, we check which package is SmartConnect default
     * on success we return Package model
     * if not found, we return null
     *
     * @return false|Package
     */
    public function getDefault()
    {
        $defaultPackageCode = $this->scopeConfig->getValue(self::DEFAULT_PACKAGE_PATH, 'website');
        $defaultConfigPackage = $this->packageRepository->loadByCode($defaultPackageCode);
        if ($defaultConfigPackage->getId()) {
            return $defaultConfigPackage;
        }


        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Package\Collection $collection */
        $collection = $this->collectionFactory->create();
        $collection->addFieldToFilter('is_default', 1);

        $defaultTranssmartPackage = $collection->fetchItem();
        return $defaultTranssmartPackage;
    }
}
