<?php
namespace Bluebirdday\TranssmartSmartConnect\Model;

/**
 * Class PackageRepository
 * @package Bluebirdday\TranssmartSmartConnect\Model
 */
class PackageRepository implements \Bluebirdday\TranssmartSmartConnect\Model\Entity\RepositoryInterface
{
    /**
     * @var PackageFactory
     */
    protected $packageFactory;

    /**
     * @var Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Package\CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Package
     */
    protected $resourceModel;

    /**
     * PackageRepository constructor.
     * @param PackageFactory $packageFactory
     * @param Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Package\CollectionFactory $collectionFactory
     * @param Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Package $resourceModel
     */
    public function __construct(
        \Bluebirdday\TranssmartSmartConnect\Model\PackageFactory $packageFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Package\CollectionFactory $collectionFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Package $resourceModel
    ) {
        $this->packageFactory = $packageFactory;
        $this->collectionFactory = $collectionFactory;
        $this->resourceModel = $resourceModel;
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Package[]
     */
    public function getList()
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Package\Collection $collection */
        $collection = $this->collectionFactory->create();
        return $collection->getItems();
    }

    /**
     * @param Package $packageModel
     */
    public function save($packageModel)
    {
        $this->resourceModel->save($packageModel);
    }

    /**
     * Load package by id
     *
     * @param integer $id
     * @return Package
     */
    public function load($id)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Package $model */
        $model = $this->packageFactory->create();
        $this->resourceModel->load($model, $id, null);
        return $model;
    }

    /**
     * Load package by code
     *
     * @param string $code
     * @return Package|null
     */
    public function loadByCode($code)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Package $model */
        $model = $this->packageFactory->create();
        $this->resourceModel->load($model, $code, 'code');
        return $model;
    }

    /**
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Package $packageModel
     */
    public function delete($packageModel)
    {
        $this->resourceModel->delete($packageModel);
    }
}
