<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Product\Attribute;

/**
 * Class Mapper
 * @package Bluebirdday\TranssmartSmartConnect\Model\Product\Attribute
 */
class Mapper
{
    const XML_PATH_MAPPING_DESCRIPTION       = 'transsmart/parcel_mapping/description';
    const XML_PATH_MAPPING_COUNTRY_OF_ORIGIN = 'transsmart/parcel_mapping/country_of_origin';
    const XML_PATH_MAPPING_HS_CODE           = 'transsmart/parcel_mapping/hs_code';
    const XML_PATH_MAPPING_REASON_OF_EXPORT  = 'transsmart/parcel_mapping/reason_of_export';

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    public $scopeConfig;

    /**
     * Mapper constructor.
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ) {
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * Map product attributes to SmartConnect shipment parameters according to backend configuration
     *
     * @param $product
     * @param null $store
     * @return array
     */
    public function getProductAdditionalFields($product, $store = null)
    {
        $line = [];
        $additionalFields = [
            'description' => $this->scopeConfig->getValue(
                self::XML_PATH_MAPPING_DESCRIPTION,
                'store',
                $store
            ),
            'countryOrigin' => $this->scopeConfig->getValue(
                self::XML_PATH_MAPPING_COUNTRY_OF_ORIGIN,
                'store',
                $store
            ),
            'hsCode' => $this->scopeConfig->getValue(
                self::XML_PATH_MAPPING_HS_CODE,
                'store',
                $store
            ),
            'reasonOfExport' => $this->scopeConfig->getValue(
                self::XML_PATH_MAPPING_REASON_OF_EXPORT,
                'store',
                $store
            ),
        ];

        foreach ($additionalFields as $field => $attributeCode) {
            if (empty($attributeCode)) {
                continue;
            }
            if ($attribute = $product->getResource()->getAttribute($attributeCode)) {
                if ($attribute->getBackendType() == 'int' && $attribute->getFrontendInput() == 'select') {
                    // dropdown or multiselect field; use the frontend label
                    $attribute->setStoreId($store->getId());
                    $value = $attribute->getFrontend()->getValue($product);
                } else {
                    $value = $product->getData($attributeCode);
                }

                if ($value) {
                    $maxLength = ($field == 'Description') ? 256 : 64;
                    $line[$field] = substr($value, 0, $maxLength);
                }
            }
        }

        return $line;
    }
}