<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Quote;


class AddressEstimationPlugin
{
    /**
     * @var \Magento\Quote\Api\CartRepositoryInterface
     */
    private $quoteRepository;

    /**
     * @var \Magento\Quote\Model\QuoteIdMaskFactory
     */
    private $quoteMaskFactory;

    /**
     * @var Estimation\AddressManager
     */
    private $addressManager;

    /**
     * AddressEstimationPlugin constructor.
     * @param \Magento\Quote\Model\QuoteIdMaskFactory $quoteMaskFactory
     * @param \Magento\Quote\Api\CartRepositoryInterface $quoteRepository
     * @param Estimation\AddressManager $addressManager
     */
    public function __construct(
        \Magento\Quote\Model\QuoteIdMaskFactory $quoteMaskFactory,
        \Magento\Quote\Api\CartRepositoryInterface $quoteRepository,
        \Bluebirdday\TranssmartSmartConnect\Model\Quote\Estimation\AddressManager $addressManager
    ) {
        $this->quoteMaskFactory = $quoteMaskFactory;
        $this->quoteRepository = $quoteRepository;
        $this->addressManager = $addressManager;
    }


    /**
     * Save address of the shipping method estimation to reuse it on rates and locations requests
     *
     * @param \Magento\Quote\Api\ShipmentEstimationInterface $subject
     * @param int $cartId
     * @param \Magento\Quote\Api\Data\AddressInterface $address
     */
    public function beforeEstimateByExtendedAddress(
        \Magento\Quote\Api\ShipmentEstimationInterface $subject,
        $cartId,
        \Magento\Quote\Api\Data\AddressInterface $address
    ) {
        if (!$address->getCity() || empty($address->getStreet())) {
            return;
        }

        if ($subject instanceof \Magento\Quote\Api\GuestShipmentEstimationInterface) {
            $quoteIdMask = $this->quoteMaskFactory->create()->load($cartId, 'masked_id');
            $cartId = $quoteIdMask->getQuoteId();
        }

        // we need to validate quote first of all
        $quote = $this->quoteRepository->getActive($cartId);

        // avoid double saving of pickup address
        $estimationAddress = $this->addressManager->getEstimationAddress($quote);
        if (!$estimationAddress) {
            /** @var \Magento\Quote\Model\Quote\Address $quoteAddress */
            $estimationAddress = $this->addressManager->create($quote);
        }

        if ($address->getExtensionAttributes() && $address->getExtensionAttributes()->getPickupMethod()) {
            $estimationAddress->setShippingMethod($address->getExtensionAttributes()->getPickupMethod());
        } else {
            $estimationAddress->setShippingMethod(null);
        }

        $this->addressManager->updateAndSave($estimationAddress, $address->getData());
    }
}
