<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Quote;

class AddressIdEstimationPlugin
{
    /**
     * @var \Magento\Quote\Api\CartRepositoryInterface
     */
    private $quoteRepository;

    /**
     * @var \Magento\Customer\Api\AddressRepositoryInterface
     */
    private $addressRepository;


    /**
     * @var Estimation\AddressManager
     */
    private $addressManager;

    /**
     * AddressEstimationPlugin constructor.
     * @param \Magento\Customer\Api\AddressRepositoryInterface $addressRepository
     * @param \Magento\Quote\Api\CartRepositoryInterface $quoteRepository
     * @param Estimation\AddressManager $addressManager
     */
    public function __construct(
        \Magento\Customer\Api\AddressRepositoryInterface $addressRepository,
        \Magento\Quote\Api\CartRepositoryInterface $quoteRepository,
        \Bluebirdday\TranssmartSmartConnect\Model\Quote\Estimation\AddressManager $addressManager
    ) {
        $this->addressRepository = $addressRepository;
        $this->quoteRepository = $quoteRepository;
        $this->addressManager = $addressManager;
    }

    /**
     * Save address of the shipping method estimation to reuse it on rates and locations requests
     *
     * @param \Magento\Quote\Api\ShippingMethodManagementInterface $subject
     * @param int $cartId
     * @param int $addressId
     */
    public function beforeEstimateByAddressId(
        \Magento\Quote\Api\ShippingMethodManagementInterface $subject,
        $cartId,
        $addressId
    ) {
        $address = $this->addressRepository->getById($addressId);

        if (!$address->getCity() || empty($address->getStreet())) {
            return;
        }

        // we need to validate quote first of all
        $quote = $this->quoteRepository->getActive($cartId);

        // avoid double saving of pickup address
        $estimationAddress = $this->addressManager->getEstimationAddress($quote);
        if (!$estimationAddress) {
            /** @var \Magento\Quote\Model\Quote\Address $quoteAddress */
            $estimationAddress = $this->addressManager->create($quote);
        }

        $this->addressManager->updateAndSave($estimationAddress, $address->__toArray());
    }
}
