<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Quote\Estimation;


class AddressManager
{
    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Quote\AddressManager
     */
    private $addressManager;

    /**
     * @var \Magento\Quote\Model\Quote\AddressFactory
     */
    private $addressFactory;

    /**
     * AddressManager constructor.
     * @param \Magento\Quote\Model\Quote\AddressFactory $addressFactory
     */
    public function __construct(
        \Magento\Quote\Model\Quote\AddressFactory $addressFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\Quote\AddressManager $addressManager

    ) {
        $this->addressFactory = $addressFactory;
        $this->addressManager = $addressManager;
    }

    /**
     * Proxy for convenience
     *
     * @return \Magento\Quote\Model\Quote\Address|null
     */
    public function getEstimationAddress($quote)
    {
       return $this->addressManager->getEstimationAddress($quote);
    }

    /**
     * Create estimation address
     *
     * @param \Magento\Quote\Api\Data\CartInterface $quote
     * @return \Magento\Quote\Model\Quote\Address
     */
    public function create($quote)
    {
        /** @var \Magento\Quote\Model\Quote\Address $quoteAddress */
        $estimationAddress = $this->addressFactory->create();
        $estimationAddress->setAddressType('estimation');
        $estimationAddress->setQuoteId($quote->getId());
        $estimationAddress->setQuote($quote);
        $quote->addAddress($estimationAddress);

        return $estimationAddress;
    }

    /**
     * @param \Magento\Quote\Model\Quote\Address $estimationAddress
     * @param array $data
     *
     * @return \Magento\Quote\Model\Quote\Address
     */
    public function updateAndSave($estimationAddress, $data)
    {
        // prevent region array to string conversion
        if (array_key_exists('region', $data) && is_array($data['region'])) {
            $regionDetails = $data['region'];
            if (array_key_exists('region_code', $regionDetails)) {
                $data['region'] = $regionDetails['region_code'];
            } else {
                $data['region'] = null;
            }
        }

        foreach ($data as $name => $value) {
            $estimationAddress->setData($name, $value);
        }

        $estimationAddress->save();

        return $estimationAddress;
    }
}
