<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Quote\QuoteRepository;

class LoadHandlerPlugin
{
    /**
     * @var \Magento\Quote\Api\Data\CartExtensionFactory
     */
    private $cartExtensionFactory;

    /**
     * LoadHandlerPlugin constructor.
     * @param \Magento\Quote\Api\Data\CartExtensionFactory $cartExtensionFactory
     */
    public function __construct(
        \Magento\Quote\Api\Data\CartExtensionFactory $cartExtensionFactory
    ){
        $this->cartExtensionFactory = $cartExtensionFactory;
    }

    /**
     * Add pickup address data on loding quote from LoadHandler
     *
     * @param \Magento\Quote\Model\QuoteRepository\LoadHandler $subject
     * @param \Closure $proceed
     * @param \Magento\Quote\Api\Data\CartInterface $quote
     * @return \Magento\Quote\Api\Data\CartInterface|mixed
     */
    public function aroundLoad(
        \Magento\Quote\Model\QuoteRepository\LoadHandler $subject,
        \Closure $proceed,
        \Magento\Quote\Api\Data\CartInterface $quote
    ) {
        $quote = $proceed($quote);

        foreach ($quote->getAllAddresses() as $address) {
            if ($address->getAddressType() === 'pickup') {
                if (!$quote->getExtensionAttributes()) {
                    $cartExtension = $this->cartExtensionFactory->create();
                    $quote->setExtensionAttributes($cartExtension);
                }
                $quote->getExtensionAttributes()->setPickupAddress($address);
            }
        }
        return $quote;
    }
}