<?php

namespace Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Carrier;

use Magento\OfflineShipping\Model\ResourceModel\Carrier\Tablerate as BaseTablerate;
use Magento\OfflineShipping\Model\ResourceModel\Carrier\Tablerate\RateQuery;

/**
 * Class Tablerate
 * @package Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Carrier
 */
class Tablerate extends BaseTablerate
{
    /**
     * @var BaseTablerate\RateQueryFactory
     */
    private $rateQueryFactory;

    /**
     * Tablerate constructor.
     * @param \Magento\Framework\Model\ResourceModel\Db\Context $context
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $coreConfig
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\OfflineShipping\Model\Carrier\Tablerate $carrierTablerate
     * @param \Magento\Framework\Filesystem $filesystem
     * @param BaseTablerate\Import $import
     * @param BaseTablerate\RateQueryFactory $rateQueryFactory
     * @param null $connectionName
     */
    public function __construct(
        \Magento\Framework\Model\ResourceModel\Db\Context $context,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\App\Config\ScopeConfigInterface $coreConfig,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\OfflineShipping\Model\Carrier\Tablerate $carrierTablerate,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\OfflineShipping\Model\ResourceModel\Carrier\Tablerate\Import $import,
        \Magento\OfflineShipping\Model\ResourceModel\Carrier\Tablerate\RateQueryFactory $rateQueryFactory,
        $connectionName = null
    )
    {
        $this->rateQueryFactory = $rateQueryFactory;
        parent::__construct($context, $logger, $coreConfig, $storeManager, $carrierTablerate, $filesystem, $import, $rateQueryFactory, $connectionName);
    }

    /**
     * Return table rate array or false by rate request
     *
     * @param \Magento\Quote\Model\Quote\Address\RateRequest $request
     * @return array|bool
     */
    public function getRates(\Magento\Quote\Model\Quote\Address\RateRequest $request)
    {
        $connection = $this->getConnection();

        $select = $connection->select()->from($this->getMainTable());
        /** @var RateQuery $rateQuery */
        $rateQuery = $this->rateQueryFactory->create(['request' => $request]);

        $rateQuery->prepareSelect($select);
        $bindings = $rateQuery->getBindings();
        $select->group('transsmart_bookingprofile_id');
        $select->limit(null);
        $result = $connection->fetchAssoc($select, $bindings);

        foreach ($result as &$row) {
            // Normalize destination zip code
            if ($row && $row['dest_zip'] == '*') {
                $row['dest_zip'] = '';
            }
        }

        return $result;
    }

}

