<?php

namespace Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Carrier\Tablerate\CSV;

use Magento\OfflineShipping\Model\ResourceModel\Carrier\Tablerate\CSV\ColumnResolver;
use Magento\OfflineShipping\Model\ResourceModel\Carrier\Tablerate\CSV\RowParser as BaseRowParser;
use Magento\OfflineShipping\Model\ResourceModel\Carrier\Tablerate\CSV\RowException;

/**
 * Class RowParser
 */
class RowParser extends BaseRowParser
{
    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileRepository
     */
    private $profileRepository;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile[]|null
     */
    private $availableProfiles = null;

    /**
     * Booking profile id column.
     */
    const COLUMN_PROFILE = 'Booking Profile';

    /**
     * RowParser constructor.
     * @param \Magento\OfflineShipping\Model\ResourceModel\Carrier\Tablerate\LocationDirectory $locationDirectory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileRepository $profileRepository
     */
    public function __construct(
        \Magento\OfflineShipping\Model\ResourceModel\Carrier\Tablerate\LocationDirectory $locationDirectory,
        \Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileRepository $profileRepository
    )
    {
        $this->profileRepository = $profileRepository;
        parent::__construct($locationDirectory);
    }

    /**
     * @inheritdoc
     */
    public function getColumns()
    {
        $columns = parent::getColumns();
        $columns[] = 'transsmart_bookingprofile_id';

        return $columns;
    }

    /**
     * @inheritdoc
     */
    public function parse(
        array $rowData,
        $rowNumber,
        $websiteId,
        $conditionShortName,
        $conditionFullName,
        ColumnResolver $columnResolver
    )
    {
        $row = parent::parse($rowData, $rowNumber, $websiteId, $conditionShortName, $conditionFullName, $columnResolver);
        $row[0]['transsmart_bookingprofile_id'] = $this->getTranssmartBookingProfileId($rowData, $rowNumber, $columnResolver);

        return $row;
    }

    /**
     * Get transsmart booking profile id.
     *
     * @param array$rowData
     * @param $rowNumber
     * @param ColumnResolver $columnResolver
     * @return int
     * @throws RowException
     */
    private function getTranssmartBookingProfileId(array $rowData, $rowNumber, ColumnResolver $columnResolver)
    {
        $profileCode = $columnResolver->getColumnValue(self::COLUMN_PROFILE, $rowData);

        $profile = $this->getBookingProfile($profileCode);
        if ($profile === null) {
            throw new RowException(__('Please correct Transsmart Booking Profile "%1" in the Row #%2', $profileCode, $rowNumber));
        }

        return $profile->getId();
    }

    /**
     * Get booking profile.
     *
     * @param string $profileCode
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile|null
     */
    private function getBookingProfile($profileCode)
    {
        $profiles = $this->getAvailableProfiles();

        return isset($profiles[$profileCode]) ? $profiles[$profileCode] : null;
    }

    /**
     * Get available profiles.
     *
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Booking\Profile[]|null
     */
    private function getAvailableProfiles()
    {
        if ($this->availableProfiles === null) {
            $profiles = [];
            foreach ($this->profileRepository->getList() as $profile) {
                $profiles[$profile->getCode()] = $profile;
            }
            $this->availableProfiles = $profiles;
        }

        return $this->availableProfiles;
    }
}