<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Servicelevel;

/**
 * Class TimeRepository
 * @package Bluebirdday\TranssmartSmartConnect\Model\Servicelevel
 */
class TimeRepository implements \Bluebirdday\TranssmartSmartConnect\Model\Entity\RepositoryInterface
{
    /**
     * @var TimeFactory
     */
    protected $timeFactory;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Servicelevel\Time\CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Servicelevel\Time
     */
    protected $resourceModel;

    /**
     * TimeRepository constructor.
     * @param TimeFactory $timeFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Servicelevel\Time\CollectionFactory $collectionFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Servicelevel\Time $resourceModel
     */
    public function __construct(
        \Bluebirdday\TranssmartSmartConnect\Model\Servicelevel\TimeFactory $timeFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Servicelevel\Time\CollectionFactory $collectionFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Servicelevel\Time $resourceModel
    ) {
        $this->timeFactory = $timeFactory;
        $this->collectionFactory = $collectionFactory;
        $this->resourceModel = $resourceModel;
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Servicelevel\Time[]
     */
    public function getList()
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Servicelevel\Time\Collection $collection */
        $collection = $this->collectionFactory->create();
        return $collection->getItems();
    }

    /**
     * @param Time $servicelevelModel
     */
    public function save($servicelevelModel)
    {
        $this->resourceModel->save($servicelevelModel);
    }

    /**
     * Load service level by id
     *
     * @param integer $id
     * @return Time
     */
    public function load($id)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Servicelevel\Time $model */
        $model = $this->timeFactory->create();
        $this->resourceModel->load($model, $id, null);
        return $model;
    }

    /**
     * Load service level by code
     *
     * @param string $code
     * @return Time
     */
    public function loadByCode($code)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Servicelevel\Time $model */
        $model = $this->timeFactory->create();
        $this->resourceModel->load($model, $code, 'code');
        return $model;
    }

    /**
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Servicelevel\Time $servicelevelModel
     */
    public function delete($servicelevelModel)
    {
        $this->resourceModel->delete($servicelevelModel);
    }
}
