<?php

namespace Bluebirdday\TranssmartSmartConnect\Model;

use Magento\Framework\Option\ArrayInterface;

/**
 * Class ShipmentStatuses
 * @package Bluebirdday\TranssmartSmartConnect\Model
 */
class ShipmentStatuses implements ArrayInterface
{
    const TRANSSMART_SHIPMENT_STATUS_NEW     = 'NEW';
    const TRANSSMART_SHIPMENT_STATUS_BOOK    = 'BOOK';
    const TRANSSMART_SHIPMENT_STATUS_LABL    = 'LABL';
    const TRANSSMART_SHIPMENT_STATUS_MANI    = 'MANI';
    const TRANSSMART_SHIPMENT_STATUS_ACCEP   = 'ACCEP';
    const TRANSSMART_SHIPMENT_STATUS_TRNS    = 'TRNS';
    const TRANSSMART_SHIPMENT_STATUS_DONE    = 'DONE';
    const TRANSSMART_SHIPMENT_STATUS_APOD    = 'APOD';
    const TRANSSMART_SHIPMENT_STATUS_REFU    = 'REFU';
    const TRANSSMART_SHIPMENT_STATUS_ERR     = 'ERR';
    const TRANSSMART_SHIPMENT_STATUS_DEL     = 'DEL';
    const TRANSSMART_SHIPMENT_STATUS_ONHOLD  = 'ONHOLD';
    const TRANSSMART_SHIPMENT_STATUS_UNKNOWN = 'UNKNOWN';
    /**
     * Get list of available statuses.
     *
     * @return array
     */
    public static function getStatuses()
    {
        return [
            self::TRANSSMART_SHIPMENT_STATUS_NEW,
            self::TRANSSMART_SHIPMENT_STATUS_BOOK,
            self::TRANSSMART_SHIPMENT_STATUS_LABL,
            self::TRANSSMART_SHIPMENT_STATUS_MANI,
            self::TRANSSMART_SHIPMENT_STATUS_ACCEP,
            self::TRANSSMART_SHIPMENT_STATUS_TRNS,
            self::TRANSSMART_SHIPMENT_STATUS_DONE,
            self::TRANSSMART_SHIPMENT_STATUS_APOD,
            self::TRANSSMART_SHIPMENT_STATUS_REFU,
            self::TRANSSMART_SHIPMENT_STATUS_ERR,
            self::TRANSSMART_SHIPMENT_STATUS_DEL,
            self::TRANSSMART_SHIPMENT_STATUS_ONHOLD,
            self::TRANSSMART_SHIPMENT_STATUS_UNKNOWN,
        ];
    }

    /**
     * Get list of editable statuses.
     *
     * @return array
     */
    public static function getEditableStatuses()
    {
        return array_diff(
            self::getStatuses(),
            [self::TRANSSMART_SHIPMENT_STATUS_LABL]
        );
    }

    /**
     * Get list of booked statuses.
     *
     * @return array
     */
    public static function getBookedStatuses()
    {
        return [
            self::TRANSSMART_SHIPMENT_STATUS_NEW,
            self::TRANSSMART_SHIPMENT_STATUS_BOOK,
            self::TRANSSMART_SHIPMENT_STATUS_LABL,
        ];
    }

    /**
     * Get list of bookable statuses.
     *
     * @return array
     */
    public static function getBookableStatuses()
    {
        return [
            self::TRANSSMART_SHIPMENT_STATUS_ERR,
            self::TRANSSMART_SHIPMENT_STATUS_REFU,
        ];
    }

    /**
     * Get list of printable statuses.
     *
     * @return array
     */
    public static function getPrintableStatuses()
    {
        // only two statuses are not printable
        return array_diff(self::getStatuses(), [
            self::TRANSSMART_SHIPMENT_STATUS_REFU,
            self::TRANSSMART_SHIPMENT_STATUS_ERR,
        ]);
    }

    /**
     * @inheritdoc
     */
    public function toOptionArray()
    {
        return [
            [
                'value' => self::TRANSSMART_SHIPMENT_STATUS_NEW,
                'label' => __('NEW'),
            ],
            [
                'value' => self::TRANSSMART_SHIPMENT_STATUS_BOOK,
                'label' => __('BOOK'),
            ],
            [
                'value' => self::TRANSSMART_SHIPMENT_STATUS_LABL,
                'label' => __('LABL'),
            ],
            [
                'value' => self::TRANSSMART_SHIPMENT_STATUS_MANI,
                'label' => __('MANI'),
            ],
            [
                'value' => self::TRANSSMART_SHIPMENT_STATUS_ACCEP,
                'label' => __('ACCEP'),
            ],
            [
                'value' => self::TRANSSMART_SHIPMENT_STATUS_TRNS,
                'label' => __('TRNS'),
            ],
            [
                'value' => self::TRANSSMART_SHIPMENT_STATUS_DONE,
                'label' => __('DONE')
            ],
            [
                'value' => self::TRANSSMART_SHIPMENT_STATUS_APOD,
                'label' => __('APOD'),
            ],
            [
                'value' => self::TRANSSMART_SHIPMENT_STATUS_REFU,
                'label' => __('REFU'),
            ],
            [
                'value' => self::TRANSSMART_SHIPMENT_STATUS_ERR,
                'label' => __('ERR'),
            ],
            [
                'value' => self::TRANSSMART_SHIPMENT_STATUS_DEL,
                'label' => __('DEL'),
            ],
            [
                'value' => self::TRANSSMART_SHIPMENT_STATUS_ONHOLD,
                'label' => __('ONHOLD'),
            ],
            [
                'value' => self::TRANSSMART_SHIPMENT_STATUS_UNKNOWN,
                'label' => __('UNKNOWN'),
            ],
        ];
    }

}