<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Shipping\Method;

use Magento\Framework\Exception\LocalizedException;

class Validator
{
    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\CarrierRepository
     */
    private $carrierRepository;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileRepository
     */
    private $profileRepository;

    /**
     * Validator constructor.
     * @param \Bluebirdday\TranssmartSmartConnect\Model\CarrierRepository $carrierRepository
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileRepository $profileRepository
     */
    public function __construct(
        \Bluebirdday\TranssmartSmartConnect\Model\CarrierRepository $carrierRepository,
        \Bluebirdday\TranssmartSmartConnect\Model\Booking\ProfileRepository $profileRepository
    ) {
        $this->carrierRepository = $carrierRepository;
        $this->profileRepository = $profileRepository;
    }

    /**
     * Validate Transsmart carrier and profile codes
     *
     * @param string $methodCode for example "PNL:PNL5"
     * @throws LocalizedException
     */
    public function validate($methodCode)
    {
        list($carrierCode, $profileCode) = explode(':', $methodCode);
        if (empty($carrierCode)) {
            throw new LocalizedException(__('Pickup carrier is empty'));
        }

        if (empty($profileCode)) {
            throw new LocalizedException(__('Pickup profile is empty'));
        }

        $carrier = $this->carrierRepository->loadByCode($carrierCode);
        if (!$carrier->getId()) {
            throw new LocalizedException(__('Invalid carrier code: %1', $carrierCode));
        }

        $profile = $this->profileRepository->loadByCode($profileCode);
        if (!$profile->getId()) {
            throw new LocalizedException(__('Invalid profile code: %1', $profileCode));
        }
    }
}
