<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Street\Address;

use Bluebirdday\TranssmartSmartConnect\Model\Adminhtml\System\Config\Source\Mapping\Street as MappingStreet;

/**
 * Class Mapper
 * @package Bluebirdday\TranssmartSmartConnect\Model\Street\Address
 */
class Mapper
{
    const XML_PATH_MAPPING_STREET   = 'transsmart/parcel_mapping/street';
    const XML_PATH_MAPPING_STREETNO = 'transsmart/parcel_mapping/streetno';
    const XML_PATH_MAPPING_STREET2  = 'transsmart/parcel_mapping/street2';

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    public $scopeConfig;

    /**
     * Mapper constructor.
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ) {
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @param integer $mappingValue
     * @param \Magento\Sales\Model\Order\Address $address
     * @return string
     */
    private function getFieldMapping($mappingValue, $address)
    {
        switch ($mappingValue) {
            case MappingStreet::NONE:
                $returnValue = '';
                break;
            case MappingStreet::FULL:
                $returnValue = trim(implode(' ', $address->getStreet()));
                break;
            default:
                $returnValue = $address->getStreetLine($mappingValue);
        }

        return $returnValue;
    }

    /**
     * @param $street
     * @return array
     */
    private function detectStreetAndHouse($street)
    {
        $finalStreet = $street;
        $finalNo = '';

        if (preg_match('/^(.*) ([0-9]+ .*)$/', $street, $matches)) {
            $finalStreet   = $matches[1];
            $finalNo = $matches[2];
        }  elseif (preg_match('/^(.*) ([0-9]+.*)$/', $street, $matches)) {
            $finalStreet   = $matches[1];
            $finalNo = $matches[2];
        }  elseif (preg_match('/^(.*) ([0-9]+.*)$/', $street, $matches)) {
            $finalStreet   = $matches[1];
            $finalNo = $matches[2];
        }

        return [$finalStreet, $finalNo];
    }

    /**
     * Get SmartConnect compatible address lines from order address object
     * Address lines mapping(configured on admin panel) is taken to account
     *
     * @param \Magento\Sales\Model\Order\Address $address
     * @param int|null $store
     * @return array
     */
    public function getStreetFields($address, $store = null)
    {
        // read config settings for street fields mapping
        $mappingStreet   = $this->scopeConfig->getValue(self::XML_PATH_MAPPING_STREET, 'store', $store);
        $mappingStreetNo = $this->scopeConfig->getValue(self::XML_PATH_MAPPING_STREETNO, 'store', $store);
        $mappingStreet2  = $this->scopeConfig->getValue(self::XML_PATH_MAPPING_STREET2, 'store', $store);

        // get values
        $street = $this->getFieldMapping($mappingStreet, $address);
        $streetNo = $this->getFieldMapping($mappingStreetNo, $address);
        $street2 = $this->getFieldMapping($mappingStreet2, $address);

        if ($mappingStreet == $mappingStreetNo) {
            list($street, $streetNo) = $this->detectStreetAndHouse($street);
        }

        // house number cannot be empty
        if ($streetNo === '') {
            $streetNo = '.';
        }

        return [
            'addressLine1' => $street,
            'addressLine2' => $street2,
            'houseNo' => $streetNo,
        ];
    }
}
