<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Street\Address;

/**
 * Class Parser
 *
 * Street address parser
 */
class Parser
{
    /**
     * Parse Magento address lines to street, house number and house number extension
     *
     * @param string[] $streetLines
     * @return string[]
     */
    public function parseStreetAddress($streetLines)
    {
        $streetName = $houseNumber = $houseExt = '';

        foreach ($streetLines as $line) {
            //first numbers found are interpreted as building number
            if (preg_match('/\d+(\-\d+)?/', $line, $numbers)) {
                $houseNumber = array_shift($numbers);
                // everything before the first number is a street name
                $streetName .= substr($line, 0, strpos($line, $houseNumber)) . ' ';
                $restOfLine = substr($line, strpos($line, $houseNumber) + strlen($houseNumber));
                $restOfLine = trim($restOfLine, "- \t\n\r\0\x0B");
                //separate single letter in the rest of line is house number extension
                if (preg_match('/^\w\b/', $restOfLine, $letter)) {
                    $houseExt = $letter[0];
                }
            } else {
                // everything BEFORE numbers is street name.
                $streetName .= $line . ' ';
            }
        }

        return [trim($streetName), $houseNumber, $houseExt];
    }

    /**
     * Split street address to lines per 64 characters
     *
     * @param string $street
     * @return array
     */
    public function splitStreetLines($street)
    {
        $parts = str_split($street, 64);
        $lines = [];
        $index = 'addressLine';
        $i = 1;
        foreach ($parts as $part) {
            $lines[$index . $i] = $part;
            if ($i >= 3) {
                return $lines;
            }
            $i++;
        }

        return $lines;
    }

    /**
     * Convert Magento streets array to Transsmart array with indexes:
     * addressLine1, addressLine2, addressLine3, houseNo
     *
     * @param string[] $streetLines
     * @return string[]
     */
    public function convertMagentoStreets(array $streetLines)
    {
        list($streetName, $houseNumber, $houseExt) = $this->parseStreetAddress($streetLines);
        $result = $this->splitStreetLines($streetName);
        $result['houseNo'] = $houseNumber . $houseExt;
        return $result;
    }
}
