<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Transsmart;

/**
 * Class SyncManagement
 * @package Bluebirdday\TranssmartSmartConnect\Model\SmartConnect
 */
class SyncManagement
{
    const MAXIMUM_SYNC_RECORDS = 100;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Transsmart\Sync\CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var SyncRepository
     */
    protected $repository;

    /**
     * @param \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Transsmart\Sync\CollectionFactory $collectionFactory
     * @param SyncRepository $repository
     */
    public function __construct(
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Transsmart\Sync\CollectionFactory $collectionFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\SyncRepository $repository
    ) {
        $this->collectionFactory = $collectionFactory;
        $this->repository = $repository;
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\Sync
     */
    public function getLatest()
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Transsmart\Sync\Collection $collection */
        $collection = $this->collectionFactory->create();
        $collection->addOrder('created_at', \Magento\Framework\Data\Collection::SORT_ORDER_DESC);
        $collection->setPageSize(1);
        return $collection->getFirstItem();
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\Sync|\Magento\Framework\DataObject|bool
     */
    public function getLatestSuccessful()
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Transsmart\Sync\Collection $collection */
        $collection = $this->collectionFactory->create();
        $collection->addFieldToFilter('status' , 'successful');
        $collection->addOrder('created_at', \Magento\Framework\Data\Collection::SORT_ORDER_DESC);
        $collection->setPageSize(1);

        return count($collection) ? $collection->getFirstItem() : false;
    }

    /**
     * Clean old sync records
     */
    public function clean()
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Transsmart\Sync\Collection $collection */
        $collection = $this->collectionFactory->create();
        $collection->addOrder('created_at', \Magento\Framework\Data\Collection::SORT_ORDER_DESC);
        $collection->setPageSize(static::MAXIMUM_SYNC_RECORDS);
        $items = $collection->getItems();

        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\Sync $theEarliestItem */
        $theEarliestItem = array_pop($items);

        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Transsmart\Sync\Collection $collection */
        $collection = $this->collectionFactory->create();
        $collection->addFieldToFilter('sync_id', ['lt' => $theEarliestItem->getId()]);
        $collection->addOrder('created_at', \Magento\Framework\Data\Collection::SORT_ORDER_ASC);
        $collection->setPageSize(static::MAXIMUM_SYNC_RECORDS);
        $itemsToRemove = $collection->getItems();

        foreach ($itemsToRemove as $item) {
            $this->repository->delete($item);
        }
    }
}
