<?php
namespace Bluebirdday\TranssmartSmartConnect\Model\Transsmart;

/**
 * Class SyncRepository
 * @package Bluebirdday\TranssmartSmartConnect\Model\SmartConnect
 */
class SyncRepository
{
    /**
     * @var SyncFactory
     */
    protected $syncFactory;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Transsmart\Sync\CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Transsmart\Sync
     */
    protected $resourceModel;

    /**
     * SyncRepository constructor.
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\SyncFactory $syncFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Transsmart\Sync\CollectionFactory $collectionFactory
     * @param \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Transsmart\Sync $resourceModel
     */
    public function __construct(
        \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\SyncFactory $syncFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Transsmart\Sync\CollectionFactory $collectionFactory,
        \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Transsmart\Sync $resourceModel
    ) {
        $this->syncFactory = $syncFactory;
        $this->collectionFactory = $collectionFactory;
        $this->resourceModel = $resourceModel;
    }

    /**
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\Sync[]
     */
    public function getList()
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\ResourceModel\Transsmart\Sync\Collection $collection */
        $collection = $this->collectionFactory->create();
        return $collection->getItems();
    }

    /**
     * @param Sync $syncModel
     */
    public function save($syncModel)
    {
        $this->resourceModel->save($syncModel);
    }

    /**
     * Load sync by id
     *
     * @param integer $id
     * @return \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\Sync
     */
    public function load($id)
    {
        /** @var \Bluebirdday\TranssmartSmartConnect\Model\Transsmart\Sync $model */
        $model = $this->syncFactory->create();
        $this->resourceModel->load($model, $id, null);
        return $model;
    }

    /**
     * @param \Bluebirdday\TranssmartSmartConnect\Model\Carrier $syncModel
     */
    public function delete($syncModel)
    {
        $this->resourceModel->delete($syncModel);
    }
}
