<?php

namespace Bluebirdday\TranssmartSmartConnect\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;

/**
 * Class ReplaceOrderShippingAddressObserver
 * @package Bluebirdday\TranssmartSmartConnect\Observer
 */
class ReplaceOrderShippingAddressObserver implements ObserverInterface
{
    /**
     * @var \Magento\Sales\Api\OrderAddressRepositoryInterface
     */
    private $addressRepository;

    /**
     * @var \Magento\Framework\Event\Manager
     */
    private $eventManager;

    /**
     * AddOrderPickupAddressObserver constructor.
     * @param \Magento\Sales\Api\OrderAddressRepositoryInterface $addressRepository
     * @param \Magento\Framework\Event\Manager $eventManager
     */
    public function __construct(
        \Magento\Sales\Api\OrderAddressRepositoryInterface $addressRepository,
        \Magento\Framework\Event\Manager $eventManager
    ) {
        $this->addressRepository = $addressRepository;
        $this->eventManager = $eventManager;
    }

    /**
     * Add pickup address information to quote
     *
     * @param EventObserver $observer
     * @return void
     */
    public function execute(EventObserver $observer)
    {
        /** @var \Magento\Quote\Api\Data\CartInterface $quote */
        $quote = $observer->getEvent()->getData('quote');
        /** @var \Magento\Sales\Api\Data\OrderInterface $quote */
        $order = $observer->getEvent()->getData('order');
        if ($quote->getExtensionAttributes() && $quote->getExtensionAttributes()->getPickupAddress()) {
            /** @var \Magento\Sales\Model\Order\Address $quotePickupAddress */
            $quotePickupAddress = $quote->getExtensionAttributes()->getPickupAddress();

            /** @var \Magento\Sales\Model\Order\Address $orderShippingAddress */
            $orderShippingAddress = $order->getShippingAddress();

            foreach ($quotePickupAddress->getData() as $name => $value) {
                if (!in_array($name, ['entity_id','address_id','id'])) {
                    $orderShippingAddress->setData($name, $value);
                }
            }
            // pickup point cannot be saved customer address
            $orderShippingAddress->setCustomerAddressId(null);

            if (!$orderShippingAddress->getEmail()) {
                $orderShippingAddress->setEmail($order->getCustomerEmail());
            }

            $orderShippingAddress->setAddressType('shipping');
            $this->addressRepository->save($orderShippingAddress);
        }
        $this->eventManager->dispatch(
            'transsmart_sales_order_process_after',
            [
                'order' => $order
            ]
        );
    }
}
