<?php

namespace Bluebirdday\TranssmartSmartConnect\Setup;

use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

class InstallSchema implements InstallSchemaInterface
{
    /**
     * {@inheritdoc}
     * @throws \Zend_Db_Exception
     */
    public function install(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $installer = $setup;

        $installer->startSetup();

        /**
         * Create table 'transsmart_carrier'
         */
        $table = $installer->getConnection()
            ->newTable($installer->getTable('transsmart_carrier'))
            ->addColumn(
                'carrier_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                'Carrier ID'
            )->addColumn(
                'code',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                32,
                ['nullable' => false, 'default' => null],
                'Carrier identifier code'
            )->addColumn(
                'name',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                255,
                ['nullable' => false, 'default' => null],
                'Carrier name'
            )->addColumn(
                'location_select',
                \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => false, 'default' => '0'],
                'Is location select available for carrier'
            )->addIndex(
                $installer->getIdxName(
                    'transsmart_carrier',
                    ['code'],
                    \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE
                ),
                ['code'],
                ['type' => \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE]
            )->setComment('Carrier Table');
        $installer->getConnection()->createTable($table);

        /**
         * Create table 'transsmart_sync'
         */
        $table = $installer->getConnection()
            ->newTable($installer->getTable('transsmart_sync'))
            ->addColumn(
                'sync_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                'Sync ID'
            )->addColumn(
                'type',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                32,
                ['nullable' => false, 'default' => null],
                'Sync Type'
            )->addColumn(
                'status',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                32,
                ['nullable' => false, 'default' => null],
                'Sync status'
            )->addColumn(
                'message',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                255,
                ['nullable' => false, 'default' => null],
                'Sync message'
            )->addColumn(
                'created_at',
                \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP,
                null,
                ['default' => new \Zend_Db_Expr('CURRENT_TIMESTAMP')],
                'Sync date and time'
            )->setComment('Transsmart Sync table');
        $installer->getConnection()->createTable($table);

        /**
         * Create table 'transsmart_mail_type'
         */
        $table = $installer->getConnection()
            ->newTable($installer->getTable('transsmart_mail_type'))
            ->addColumn(
                'mailtype_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                'Mail Type ID'
            )->addColumn(
                'code',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                32,
                ['nullable' => false, 'default' => null],
                'The mail type id code'
            )->addColumn(
                'description',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                500,
                ['nullable' => true],
                'Description of the mail type'
            )->addColumn(
                'is_default',
                \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => false, 'default' => '0'],
                'Is this the default mail type'
            )->addIndex(
                $installer->getIdxName(
                    'transsmart_mail_type',
                    ['code'],
                    \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE
                ),
                ['code'],
                ['type' => \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE]
            )->setComment('Mail Type Table');
        $installer->getConnection()->createTable($table);

        /**
         * Create table 'transsmart_service_level_time'
         */
        $table = $installer->getConnection()
            ->newTable($installer->getTable('transsmart_service_level_time'))
            ->addColumn(
                'service_level_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                'Service Level ID'
            )->addColumn(
                'code',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                32,
                ['nullable' => false, 'default' => null],
                'The service level code'
            )->addColumn(
                'description',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                500,
                ['nullable' => true],
                'Description of the service level'
            )->addColumn(
                'is_default',
                \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => false, 'default' => '0'],
                'Is this the default service level'
            )->addIndex(
                $installer->getIdxName(
                    'transsmart_service_level_time',
                    ['code'],
                    \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE
                ),
                ['code'],
                ['type' => \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE]
            )->setComment('Service Level Time Table');
        $installer->getConnection()->createTable($table);

        /**
         * Create table 'transsmart_service_level_other'
         */
        $table = $installer->getConnection()
            ->newTable($installer->getTable('transsmart_service_level_other'))
            ->addColumn(
                'service_level_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                'Service Level ID'
            )->addColumn(
                'code',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                32,
                ['nullable' => false, 'default' => null],
                'The service level code'
            )->addColumn(
                'description',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                500,
                ['nullable' => true],
                'Description of the service level'
            )->addColumn(
                'is_default',
                \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => false, 'default' => '0'],
                'Is this the default service level'
            )->addIndex(
                $installer->getIdxName(
                    'transsmart_service_level_other',
                    ['code'],
                    \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE
                ),
                ['code'],
                ['type' => \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE]
            )->setComment('Service Level Other Table');
        $installer->getConnection()->createTable($table);

        /**
         * Create table 'transsmart_costcenter'
         */
        $table = $installer->getConnection()
            ->newTable($installer->getTable('transsmart_costcenter'))
            ->addColumn(
                'costcenter_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                'Costcenter ID'
            )->addColumn(
                'code',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                32,
                ['nullable' => false, 'default' => null],
                'Costcenter id code'
            )->addColumn(
                'description',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                500,
                ['nullable' => true],
                'Description of the costcenter'
            )->addColumn(
                'is_default',
                \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => false, 'default' => '0'],
                'Is this the default costcenter'
            )->addIndex(
                $installer->getIdxName(
                    'transsmart_costcenter',
                    ['code'],
                    \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE
                ),
                ['code'],
                ['type' => \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE]
            )->setComment('Costcenter Table');
        $installer->getConnection()->createTable($table);

        /**
         * Create table 'transsmart_incoterm'
         */
        $table = $installer->getConnection()
            ->newTable($installer->getTable('transsmart_incoterm'))
            ->addColumn(
                'incoterm_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                'Incoterm ID'
            )->addColumn(
                'code',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                32,
                ['nullable' => false, 'default' => null],
                'The incoterm id code'
            )->addColumn(
                'description',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                500,
                ['nullable' => true],
                'Description of the incoterm'
            )->addColumn(
                'is_default',
                \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => false, 'default' => '0'],
                'Is this the default incoterm'
            )->addIndex(
                $installer->getIdxName(
                    'transsmart_incoterm',
                    ['code'],
                    \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE
                ),
                ['code'],
                ['type' => \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE]
            )
            ->setComment('Incoterm Table');
        $installer->getConnection()->createTable($table);

        /**
         * Create table 'transsmart_package'
         */
        $table = $installer->getConnection()
            ->newTable($installer->getTable('transsmart_package'))
            ->addColumn(
                'package_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                'Package ID'
            )->addColumn(
                'code',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                32,
                ['nullable' => false, 'default' => null],
                'Package code'
            )->addColumn(
                'description',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                500,
                ['nullable' => true],
                'Description of the package'
            )->addColumn(
                'is_default',
                \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
                null,
                ['unsigned' => true, 'nullable' => false, 'default' => '0'],
                'Is this the default package to use'
            )->addColumn(
                'type',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                255,
                ['nullable' => false],
                'Type of the package'
            )->addColumn(
                'linear_uom',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                255,
                ['nullable' => false],
                'Unit of Measurment for the size'
            )->addColumn(
                'mass_uom',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                255,
                ['nullable' => false],
                'Unit of Measurment for the weight'
            )->addColumn(
                'length',
                \Magento\Framework\DB\Ddl\Table::TYPE_FLOAT,
                null,
                ['nullable' => false],
                'Length of the package'
            )->addColumn(
                'width',
                \Magento\Framework\DB\Ddl\Table::TYPE_FLOAT,
                null,
                ['nullable' => false],
                'Width of the package'
            )->addColumn(
                'height',
                \Magento\Framework\DB\Ddl\Table::TYPE_FLOAT,
                null,
                ['nullable' => false],
                'Height of the package'
            )->addColumn(
                'weight',
                \Magento\Framework\DB\Ddl\Table::TYPE_FLOAT,
                null,
                ['nullable' => false],
                'Weight of the package'
            )->addIndex(
                $installer->getIdxName(
                    'transsmart_package',
                    ['code'],
                    \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE
                ),
                ['code'],
                ['type' => \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE]
            )->setComment('Package Table');
        $installer->getConnection()->createTable($table);

        /**
         * Create table 'transsmart_booking_profile'
         */
        $table = $installer->getConnection()
            ->newTable($installer->getTable('transsmart_booking_profile'))
            ->addColumn(
                'profile_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                'Booking Profile Id'
            )->addColumn(
                'code',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                32,
                ['nullable' => false, 'default' => null],
                'Booking profile code'
            )->addColumn(
                'description',
                \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                500,
                ['nullable' => true],
                'Description of a booking profile'
            )->addColumn(
                'carrier_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['unsigned' => true, 'nullable' => false],
                'Carrier Reference'
            )->addColumn(
                'slt_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['unsigned' => true, 'nullable' => true],
                'Service Level Time Id Reference'
            )->addColumn(
                'slo_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['unsigned' => true, 'nullable' => true],
                'Service Level Other Id Reference'
            )->addColumn(
                'incoterm_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['unsigned' => true, 'nullable' => true],
                'Incoterm id reference'
            )->addColumn(
                'costcenter_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['unsigned' => true, 'nullable' => true],
                'Costcenter Id Reference'
            )->addColumn(
                'mailtype_id',
                \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                null,
                ['unsigned' => true, 'nullable' => true],
                'Mail Type Id Reference'
            )->addIndex(
                $installer->getIdxName(
                    'transsmart_booking_profile',
                    ['code'],
                    \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE
                ),
                ['code'],
                ['type' => \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE]
            )->addForeignKey(
                $installer->getFkName(
                    'transsmart_booking_profile',
                    'carrier_id',
                    'transsmart_carrier',
                    'carrier_id'
                ),
                'carrier_id',
                $installer->getTable('transsmart_carrier'),
                'carrier_id',
                \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE
            )->addForeignKey(
                $installer->getFkName(
                    'transsmart_booking_profile',
                    'slt_id',
                    'transsmart_service_level_time',
                    'service_level_id'
                ),
                'slt_id',
                $installer->getTable('transsmart_service_level_time'),
                'service_level_id',
                \Magento\Framework\DB\Ddl\Table::ACTION_SET_NULL
            )->addForeignKey(
                $installer->getFkName(
                    'transsmart_booking_profile',
                    'slo_id',
                    'transsmart_service_level_other',
                    'service_level_id'
                ),
                'slo_id',
                $installer->getTable('transsmart_service_level_other'),
                'service_level_id',
                \Magento\Framework\DB\Ddl\Table::ACTION_SET_NULL
            )->addForeignKey(
                $installer->getFkName(
                    'transsmart_booking_profile',
                    'incoterm_id',
                    'transsmart_incoterm',
                    'incoterm_id'
                ),
                'incoterm_id',
                $installer->getTable('transsmart_incoterm'),
                'incoterm_id',
                \Magento\Framework\DB\Ddl\Table::ACTION_SET_NULL
            )->addForeignKey(
                $installer->getFkName(
                    'transsmart_booking_profile',
                    'costcenter_id',
                    'transsmart_costcenter',
                    'costcenter_id'
                ),
                'costcenter_id',
                $installer->getTable('transsmart_costcenter'),
                'costcenter_id',
                \Magento\Framework\DB\Ddl\Table::ACTION_SET_NULL
            )->addForeignKey(
                $installer->getFkName(
                    'transsmart_booking_profile',
                    'mailtype_id',
                    'transsmart_mail_type',
                    'mailtype_id'
                ),
                'mailtype_id',
                $installer->getTable('transsmart_mail_type'),
                'mailtype_id',
                \Magento\Framework\DB\Ddl\Table::ACTION_SET_NULL
            )->setComment('Booking Profiles Table');
        $installer->getConnection()->createTable($table);

        /**
         * Extending default magento tablerate schema.
         */
        $installer->getConnection()->addColumn(
            $installer->getTable('shipping_tablerate'),
            'transsmart_bookingprofile_id',
            [
                'type' => \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                'nullable' => true,
                'unsigned' => true,
                'comment' => 'Transsmart Booking Profile Id'
            ]
        );
        $installer->getConnection()->addForeignKey(
            $setup->getFkName(
                'shipping_tablerate',
                'transsmart_bookingprofile_id',
                $setup->getTable('transsmart_booking_profile'),
                'profile_id'
            ),
            $setup->getTable('shipping_tablerate'),
            'transsmart_bookingprofile_id',
            $setup->getTable('transsmart_booking_profile'),
            'profile_id',
            \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE
        );
        $installer->getConnection()->dropIndex(
            $setup->getTable('shipping_tablerate'),
            $installer->getIdxName(
                'shipping_tablerate',
                ['website_id', 'dest_country_id', 'dest_region_id', 'dest_zip', 'condition_name', 'condition_value'],
                \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE
            )
        );
        $installer->getConnection()->addIndex(
            $setup->getTable('shipping_tablerate'),
            $installer->getIdxName(
                'shipping_tablerate',
                ['website_id', 'dest_country_id', 'dest_region_id', 'dest_zip', 'condition_name', 'condition_value', 'transsmart_bookingprofile_id'],
                \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE
            ),
            ['website_id', 'dest_country_id', 'dest_region_id', 'dest_zip', 'condition_name', 'condition_value', 'transsmart_bookingprofile_id'],
            \Magento\Framework\DB\Adapter\AdapterInterface::INDEX_TYPE_UNIQUE
        );

        $installer->endSetup();
    }
}
