define([
    'Magento_Checkout/js/model/resource-url-manager',
    'Magento_Checkout/js/model/quote',
    'mage/storage',
    'Magento_Checkout/js/model/shipping-service',
    'Magento_Checkout/js/model/shipping-rate-registry',
    'Magento_Checkout/js/model/error-processor',
    'Bluebirdday_TranssmartSmartConnect/js/model/location-model'
], function (resourceUrlManager, quote, storage, shippingService, rateRegistry, errorProcessor, locationModel) {
    'use strict';

    var lastRequestDate = 0;
    return {
        /**
         * Get shipping rates for specified address.
         * @param {Object} address
         */
        getRates: function (address) {
            var cache, serviceUrl, payload;

            shippingService.isLoading(true);
            cache = rateRegistry.get(address.getCacheKey());
            serviceUrl = resourceUrlManager.getUrlForEstimationShippingMethodsForNewAddress(quote);
            var addressData = {
                'street': address.street,
                    'city': address.city,
                    'region_id': address.regionId,
                    'region': address.region,
                    'country_id': address.countryId,
                    'postcode': address.postcode,
                    'email': address.email,
                    'customer_id': address.customerId,
                    'firstname': address.firstname,
                    'lastname': address.lastname,
                    'middlename': address.middlename,
                    'prefix': address.prefix,
                    'suffix': address.suffix,
                    'vat_id': address.vatId,
                    'company': address.company,
                    'telephone': address.telephone,
                    'fax': address.fax,
                    'custom_attributes': address.customAttributes,
                    'save_in_address_book': address.saveInAddressBook
            };

            var locationUpdate = 0;
            var locationData = locationModel.selectedLocationData();
            if (locationData && locationData.method_code) {
                addressData.extension_attributes = {
                    pickup_method: 'transsmartpickup_' + locationData.method_code
                };
                if (locationData.update) {
                    locationUpdate = locationData.update;
                }
            }
            payload = JSON.stringify({
                address: addressData
                }
            );

            if (cache && lastRequestDate >= locationUpdate) {
                shippingService.setShippingRates(cache);
                shippingService.isLoading(false);
            } else {
                lastRequestDate = Date.now();
                storage.post(
                    serviceUrl, payload, false
                ).done(function (result) {
                    rateRegistry.set(address.getCacheKey(), result);
                    shippingService.setShippingRates(result);
                }).fail(function (response) {
                    shippingService.setShippingRates([]);
                    errorProcessor.process(response);
                }).always(function () {
                    shippingService.isLoading(false);
                });
            }
        }
    };
});
