/**
 * @see Magento_Checkout/js/view/shipping-information/list
 *
 * We display pickup point address instead of shipping address for pickup shipping method
 */
define([
    'jquery',
    'ko',
    'mageUtils',
    'uiComponent',
    'uiLayout',
    'Magento_Checkout/js/model/quote',
    'Magento_Customer/js/customer-data',
    'Bluebirdday_TranssmartSmartConnect/js/model/location-model',
    'Magento_Checkout/js/view/shipping'
], function ($, ko, utils, Component, layout, quote, customerData, locationModel) {
    'use strict';

    var defaultRendererTemplate = {
        parent: '${ $.$data.parentName }',
        name: '${ $.$data.name }',
        component: 'Bluebirdday_TranssmartSmartConnect/js/view/shipping-information/address-renderer/default'
    };

    return Component.extend({
        defaults: {
            template: 'Magento_Checkout/shipping-information/list',
            rendererTemplates: {}
        },

        /** @inheritdoc */
        initialize: function () {
            var self = this;

            this._super()
                .initChildren();

            quote.shippingAddress.subscribe(function (address) {
                self.createRendererComponent(address);
            });

            // we need to renderer component when pickup shipping method was selected
            quote.shippingMethod.subscribe(function (method) {
                var address = quote.shippingAddress();
                self.createRendererComponent(address);
            });

            // when pickup location is selected we need to update displayed data
            locationModel.pickupAddress.subscribe(function (address) {
                self.createRendererComponent(address);
            });

            return this;
        },

        /** @inheritdoc */
        initConfig: function () {
            this._super();
            // the list of child components that are responsible for address rendering
            this.rendererComponents = {};

            return this;
        },

        /** @inheritdoc */
        initChildren: function () {
            return this;
        },

        /**
         * Create new component that will render given address in the address list
         *
         * @param {Object} address
         */
        createRendererComponent: function (address) {
            var rendererTemplate, templateData, rendererComponent;

            // when pickup method was selected we show pickup point address as shipping address
            var checkoutData = customerData.get('checkout-data')();
            var addressType = address.getType();
            var combinedAddress = $.extend({}, address);
            if (locationModel.pickupAddress() &&
                quote.shippingMethod() &&
                quote.shippingMethod()['carrier_code'] === 'transsmartpickup'
            ) {
                combinedAddress = $.extend(combinedAddress, checkoutData.pickupAddress);
            }



            $.each(this.rendererComponents, function (index, component) {
                component.visible(false);
            });

            if (this.rendererComponents[addressType]) {
                this.rendererComponents[addressType].address(combinedAddress);
                this.rendererComponents[addressType].visible(true);
            } else {
                // rendererTemplates are provided via layout
                rendererTemplate = addressType != undefined && this.rendererTemplates[addressType] != undefined ? //eslint-disable-line
                    utils.extend({}, defaultRendererTemplate, this.rendererTemplates[addressType]) :
                    defaultRendererTemplate;
                templateData = {
                    parentName: this.name,
                    name: addressType
                };

                rendererComponent = utils.template(rendererTemplate, templateData);
                utils.extend(
                    rendererComponent,
                    {
                        address: ko.observable(combinedAddress),
                        visible: ko.observable(true)
                    }
                );
                layout([rendererComponent]);
                this.rendererComponents[addressType] = rendererComponent;
            }
        }
    });
});
